<?php

class Scalapay_Scalapay_Block_Form_Scalapay extends Scalapay_Scalapay_Block_Form_Abstract
{
    const TITLE_TEMPLATE_SELECTOR_ID = 'scalapay_checkout_scalapay_headline';
    const DETAIL_TEMPLATE_SELECTOR_ID = 'scalapay_checkout_scalapay_form';
    const TEMPLATE_OPTION_TITLE_CUSTOM = 'Scalapay/checkout/title_custom.phtml';
    const TEMPLATE_OPTION_DETAILS_CUSTOM = 'Scalapay/form/scalapay_custom.phtml';
    protected $helper;

    protected function _construct()
    {
        parent::_construct();
        $this->helper = Mage::helper('scalapay');
        $block = Mage::getConfig()->getBlockClassName('core/template');
        $block = new $block();
        $block->setTemplateHelper($this);
        $block->setTemplate(self::TEMPLATE_OPTION_TITLE_CUSTOM);
        if (!$this->helper->getConfigData('show_installment_description')) {
            $this->setTemplate(self::TEMPLATE_OPTION_DETAILS_CUSTOM);
        } else {
            $this->setTemplate('');
        }
        $this->setMethodTitle('')
            ->setMethodLabelAfterHtml($block->toHtml());
    }

    public function getInstallmentPrice()
    {
        $helper = Mage::helper('scalapay');
        $configData = $helper->getScalapayConfig();
        $numberOfPayments = $configData["numberOfPayments"];
        $total = Mage::getSingleton('checkout/session')->getQuote()->getGrandTotal();

        if ($numberOfPayments > 0) {
            $total = $total / $numberOfPayments;
        }
        $total = round($total, 2);

        return $helper->getScalapayFormatedPrice($total);
    }

    public function getPaymentMethodDetails()
    {
        $config = $this->getCommonConfiguration();
        $config['template'] = $this->getScalapayDetailTemplate();
        $config['cssSelector'] = '#' . $this->getDetailsTemplateId();

        return $config;
    }

    public function getTitleConfiguration()
    {
        $config = $this->getCommonConfiguration();
        $config['template'] = $this->getScalapayTitleTemplate();
        $config['cssSelector'] = '#' . $this->getTitleTemplateId();

        return $config;
    }

    public function getDetailsTemplateId()
    {
        return self::DETAIL_TEMPLATE_SELECTOR_ID;
    }

    public function getTitleTemplateId()
    {
        return self::TITLE_TEMPLATE_SELECTOR_ID;
    }

    public function getScalapayLogo()
    {
        $helper = Mage::helper('scalapay');
        $checkout_logo = $helper->getScalapayCheckoutPageLogo();
        $product_logo = $helper->getScalapayLogo();
        $logo = Mage::getBaseUrl(Mage_Core_Model_Store::URL_TYPE_SKIN) .
        "frontend/base/default/scalapay/images/" . $checkout_logo;

        if ($helper->isFirecheckoutEnabled()) {
            $img_src = '<img id="scalapay_text_firecheckout_view" width="95px" src="' . $logo . '" />';
        } else {
            $img_src = '<img id="scalapay_text_checkout_view"  width="95px"  src="' . $logo . '" />';
        }

        return $img_src;
    }

    private function getCommonConfiguration()
    {
        $helper = Mage::helper('scalapay');
        $configData = $helper->getScalapayConfig();
        $numberOfPayments = $configData["numberOfPayments"];
        return array(
            'logoReplace' => '<logo>',
            'scalapayLogo' => $this->getScalapayLogo(),
            'priceReplace' => '<price>',
            'price' => $this->getInstallmentPrice(),
            'nopReplace' => '<nop>',
            'scalapayNop' => $numberOfPayments
        );
    }

    private function getScalapayDetailTemplate()
    {
        return $this->helper->getConfigData('installment_description');
    }

    private function getScalapayTitleTemplate()
    {
        return $this->helper->getConfigData('installment_checkout_description');
    }

    public function getMethodTitle()
    {
        $defaultCheckout = Mage::helper('core')->isModuleEnabled("Mage_Checkout");
        if (!$defaultCheckout) {
            $helper = Mage::helper('scalapay');
            $getCurrentQuote = $helper->getQuote();
            $configData = $helper->getScalapayConfig();
            $grandTotal = $getCurrentQuote->getGrandTotal();
            $numberOfPayments = $configData["numberOfPayments"];
            if ($numberOfPayments > 0) {
                $scalapayInstallementPrice = round($grandTotal / $numberOfPayments, 2);
            } else {
                $scalapayInstallementPrice = 0;
            }

            return $helper->getScalapayCheckoutPageText($scalapayInstallementPrice);
        } else {
            return "";
        }
    }
}
