<?php

class Scalapay_Scalapay_Block_ProductInstalments extends Mage_Core_Block_Template
{
    /**
     * Get current product object
     *
     * @return object
     *
     */
    public function getCurrentProduct()
    {
        return Mage::registry('current_product');
    }

    /**
     * Get Scalapay configurations
     *
     * @return Array
     *
     */
    public function ScalapayConfig()
    {
        $helper = Mage::helper('scalapay');
        return $helper->getScalapayConfig();
    }

    public function getCssSelectors()
    {
        $helper = Mage::helper('scalapay');
        $selectors = $helper->getConfigData('product_page_price_block_selectors');
        return explode("\n", $selectors);
    }

    public function getScalapayEnabled()
    {
        $helper = Mage::helper('scalapay');
        //Do not show if product of type virtual
        $currentProduct = $this->getCurrentProduct();
        $productPrice = $currentProduct->getFinalPrice();
        $productId = $currentProduct->getId();
        $isActive = $helper->isSclapayActive($productPrice, [$productId]);

        return $isActive;
    }

    public function getHtmlTemplate()
    {
        $helper = Mage::helper('scalapay');
        $result = $helper->getConfigData('installment_copy_description');
        return $result;
    }

    public function getTemplateCss()
    {
        $helper = Mage::helper('scalapay');
        $result = $helper->getConfigData('installment_copy_description_css');
        return $result;
    }

    public function getScalapayLogo()
    {
        $logo = Mage::getBaseUrl(Mage_Core_Model_Store::URL_TYPE_SKIN) .
        "frontend/base/default/scalapay/images/scalapay-logo-black.png";
        $img_src = '<img id="scalapay_text_product_view" src="' . $logo . '" />';

        return $img_src;
    }

    public function getJsConfig()
    {
        $helper = Mage::helper('scalapay');
        $scalapay_config = $helper->getScalapayConfig();
        $template = $helper->__($this->getHtmlTemplate());

        $minPriceLimit = $scalapay_config['minimumAmount'];
        $maxPriceLimit = $scalapay_config['maximumAmount'];
        $installmentsAmount = $scalapay_config['numberOfPayments'];
        //
        return [
            'selectors' => $this->getCssSelectors(),
            'template' => $template,
            'template_css' => $this->getTemplateCss(),
            'logo_css' => '',
            'priceReplace' => '<price>',
            'emojiReplace' => '<emoji>',
            'logoReplace' => '<logo>',
            'scalapay_logo' => $this->getScalapayLogo(),
            'minPriceLimit' => $minPriceLimit,
            'maxPriceLimit' => $maxPriceLimit,
            'installmentsAmount' => $installmentsAmount,
            'scalapayEnabled' => $this->getScalapayEnabled(),
            'priceFormat' => Mage::app()->getLocale()->getJsPriceFormat(),
            'currencySymbol' => Mage::app()->getLocale()->currency(
                Mage::app()->getStore()->getCurrentCurrencyCode()
            )->getSymbol(),
            'className' => 'scalapay-installments-amount'
        ];
    }
}
