<?php

class Scalapay_Scalapay_Helper_Data extends Mage_Core_Helper_Abstract
{
    public $checkoutSession;
    protected $shippingMethod;
    protected $ws_url;
    protected $ws_password;
    protected $scalapayConfig;
    protected $payin4Config;
    protected $paylaterConfig;

    /**
     * Set Scalapay credentials
     */
    public function getScalapayConnection()
    {
        $live_mode = $this->getConfigData('live_mode');

        if ($live_mode) {
            $this->ws_url = $this->getConfigData('production_url');
            $this->ws_password = $this->getConfigData('production_api_password');
        } else {
            $this->ws_url = $this->getConfigData('test_url');
            $this->ws_password = $this->getConfigData('test_api_password');
        }

        $this->checkoutSession = Mage::getSingleton('checkout/session');
    }

    /**
     * Read response for Scalapay curl request
     *
     * @return Array
     *
     */
    public function getScalapayConfig()
    {
        if (!isset($this->scalapayConfig)) {
            $this->scalapayConfig = [
                "minimumAmount" => $this->getConfigData('min_amount'),
                "maximumAmount" => $this->getConfigData('max_amount'),
                "numberOfPayments" => $this->getConfigData('number_of_payments'),
                "promotionUrl" => "https://promotion.scalapay.it/popup/default/",
                "locales" => ["it","fr","en"]
            ];
        }

        return $this->scalapayConfig;
    }

    /**
     * Read response for Scalapay curl request
     *
     * @return Array
     *
     */
    public function getPayin4Config()
    {
        if (!isset($this->payin4Config)) {
            $this->payin4Config = [
                "minimumAmount" => $this->getPayin4ConfigData('min_amount'),
                "maximumAmount" => $this->getPayin4ConfigData('max_amount'),
                "numberOfPayments" => $this->getPayin4ConfigData('number_of_payments'),
                "promotionUrl" => "https://promotion.scalapay.it/popup/default/",
                "locales" => ["it","fr","en"]
            ];
        }

        return $this->payin4Config;
    }

    /**
     * Read response for Scalapay curl request
     *
     * @return Array
     *
     */
    public function getPaylaterConfig()
    {
        if (!isset($this->paylaterConfig)) {
            $this->paylaterConfig = [
                "minimumAmount" => $this->getPaylaterConfigData('min_amount'),
                "maximumAmount" => $this->getPaylaterConfigData('max_amount'),
                "numberOfPayments" => $this->getPaylaterConfigData('number_of_payments'),
                "numberOfDays" => $this->getPaylaterConfigData('number_of_days'),
                "promotionUrl" => "https://promotion.scalapay.it/popup/default/",
                "locales" => ["it","fr","en"]
            ];
        }

        return $this->paylaterConfig;
    }

    /**
     * Set Scalapay default configuration
     */
    protected function updateScalapayConfig()
    {
        $data = [];
        $urlKey = "v2/configurations";
        $result = $this->getCurlHandler($urlKey, $data, "GET");
        $error = $result["error"];
        $response = $result["response"];

        if ($response != "Unauthorized") {
            if (!$error) {
                $res = json_decode($response, true);
                if (isset($res)) {
                    if (isset($res["minimumAmount"]["amount"])) {
                        $this->setConfigData(
                            'scalapay/settings/min_amount',
                            $res["minimumAmount"]["amount"]
                        );
                    }
                    if (isset($res["maximumAmount"]["amount"])) {
                        $this->setConfigData(
                            'scalapay/settings/max_amount',
                            $res["maximumAmount"]["amount"]
                        );
                    }
                    if (isset($res["numberOfPayments"])) {
                        $this->setConfigData(
                            'scalapay/settings/number_of_payments',
                            $res["numberOfPayments"]["amount"]
                        );
                    }
                }
            }
        }
    }

    /**
     * Set Cookie value
     *
     * @param string $name
     * @param string $value
     */
    public function setCookie($name, $value)
    {
        if ($name != null) {
            $cacheTime = $this->getConfigData('configuration_cache_time');
            $cacheTime = ($cacheTime != null && $cacheTime > 0) ? $cacheTime : 24;
            $cookieTime = $cacheTime * 3600;
            $cookie = Mage::getSingleton('core/cookie');
            $cookie->set($name, $value, $cookieTime, '/');
        }
    }

    /**
     * Read Cookie value
     *
     * @param string $name
     *
     * @return string
     */
    public function getCookie($name)
    {
        $value = "";
        if ($name != null) {
            $cookie = Mage::getSingleton('core/cookie');
            $value = $cookie->get($name);
        }

        return $value;
    }

    /**
     * Return promotion url on the base of language
     *
     * @return string
     */
    public function getPromotionUrl()
    {
        $configData = $this->getScalapayConfig();
        $allowedLanguages = $configData["locales"];
        $locales = Mage::getStoreConfig('general/locale/code', Mage::app()->getStore()->getId());
        $promotionUrl = $configData["promotionUrl"];
        if ($locales != null) {
            $language = substr($locales, 0, 2);
            if ($configData["promotionUrl"] != "") {
                if (in_array($language, $allowedLanguages)) {
                    $promotionUrl = $configData["promotionUrl"] . $language . "/";
                }
            }
        }

        return $promotionUrl;
    }

    /**
     * Read response for Scalapay curl request while placing order
     *
     * @return checkoutUrl
     * @error Array
     *
     */
    public function scalapayOrder($params = array())
    {
        $json_array = [];
        $this->writeLog("scalapay order params: ");
        $this->writeLog($params);
        $getCurrentQuote = $this->getOrder();
        $getAllitems = $getCurrentQuote->getAllItems();
        $DiscountTotal = 0;
        $currency = $getCurrentQuote->getQuoteCurrencyCode();
        $paymentMethod = $getCurrentQuote->getPayment()->getMethod();

        // Get cart total items and total quantity
        $totalItems = $getCurrentQuote->getItemsCount();
        $totalQuantity = $getCurrentQuote->getItemsQty();

        //get cart subtotal and grand total
        $subTotal = $getCurrentQuote->getSubtotal();
        $grandTotal = $getCurrentQuote->getGrandTotal();

        //get cartbilling and shipping addresses
        $shipping = $this->getShippingInfo();
        $billing = $this->getBillingInfo();
        $this->writeLog("Shipping Data: ");
        $this->writeLog($shipping->getData());
        $this->writeLog("Billing Data: ");
        $this->writeLog($billing->getData());

        if (
            Mage_Checkout_Model_Type_Onepage::METHOD_REGISTER ==
            Mage::getSingleton('checkout/type_onepage')->getQuote()->getCheckoutMethod() &&
            !Mage::getSingleton('checkout/type_onepage')->getQuote()->getCustomerId()
        ) {
            $client = Mage::getModel('customer/customer');
            $client->setWebsiteId(Mage::app()->getWebsite()->getId());
            $customerEmail = "";
            if ($billing->getEmail() != "") {
                $customerEmail = $billing->getEmail();
            }
            if ($customerEmail == "-") {
                $customerEmail = "";
            }
            if ($customerEmail == "") {
                $customerEmail = $getCurrentQuote->getCustomerEmail();
            }
            if ($customerEmail == "" && isset($params['billing']['email'])) {
                $customerEmail = $params['billing']['email'];
            }
            if ($customerEmail == "" && isset($params["email"])) {
                $customerEmail = $params["email"];
            }
            $client->loadByEmail($customerEmail);
            if ($client->getId()) {
                $return_error_arrays = [];
                $return_error_arrays["error"] = 1;
                $return_error_arrays["message"] = $this->__(
                    'There is already a customer registered using this email address. Please login using this email address or enter a different email address to register your account.'
                );
                $this->writeLog($return_error_arrays);

                return $return_error_arrays;
            }
        }

        //Billing information
        $city = $billing->getCity();
        if ($city == "-") {
            $city = "";
        }
        if ($city == "" && isset($params['billing']['city'])) {
            $city = $params['billing']['city'];
            $billing->setCity($city);
        }

        $post_code = $billing->getPostcode();
        if ($post_code == "-") {
            $post_code = "";
        }
        if ($post_code == "" && isset($params['billing']['postcode'])) {
            $post_code = $params['billing']['postcode'];
            $billing->setPostcode($post_code);
        }

        $email = $billing->getEmail();
        if ($email == "-") {
            $email = "";
        }
        if ($email == "") {
            $email = $getCurrentQuote->getCustomerEmail();
        }
        if ($email == "" && isset($params['billing']['email'])) {
            $email = $params['billing']['email'];
        }
        if ($email == "" && isset($params["email"])) {
            $email = $params["email"];
        }

        if (isset($email) && $email != "") {
            $billing->setEmail($email);
        }

        $telephone = $billing->getTelephone();
        if ($telephone == "-") {
            $telephone = "";
        }
        if ($telephone == "" && isset($params['billing']['telephone'])) {
            $telephone = $params['billing']['telephone'];
        }
        if ($telephone != "") {
            $billing->setTelephone($telephone);
        }

        $state = $billing->getRegion();
        if ($state == "-") {
            $state = "";
        }
        if ($state == "" && isset($params['billing']['region'])) {
            $state = $params['billing']['region'];
            $billing->setRegion($state);
        }

        $region_id = $billing->getRegionId();
        if ($region_id == "-") {
            $region_id = "";
        }
        if ($region_id == "" && isset($params['billing']['region_id'])) {
            $region_id = $params['billing']['region_id'];
            $billing->setRegionId($region_id);
        }

        $street = $billing->getStreet();
        $street_address0 = $street[0];
        if ($street_address0 == "" && isset($params['billing']['street'][0])) {
            $street_address0 = $params['billing']['street'][0];
        }
        $street_address1 = "";
        if (isset($street[1])) {
            $street_address1 = $street[1];
            if ($street_address1 == "" && isset($params['billing']['street'][1])) {
                $street_address1 = $params['billing']['street'][1];
            }
        }
        $billing->setStreet(
            array(
                0 => $street_address0,
                1 => $street_address1
            )
        );

        $firstname = $billing->getFirstname();
        if ($firstname == "-") {
            $firstname = "";
        }
        if ($firstname == "" && isset($params['billing']['firstname'])) {
            $firstname = $params['billing']['firstname'];
        }
        if ($firstname == "") {
            $firstname = $getCurrentQuote->getCustomerFirstname();
        }
        $billing->setFirstname($firstname);

        $last_name = $billing->getLastname();
        if ($last_name == "-") {
            $last_name = "";
        }
        if ($last_name == "" && isset($params['billing']['lastname'])) {
            $last_name = $params['billing']['lastname'];
        }
        if ($last_name == "") {
            $last_name = $getCurrentQuote->getCustomerLastname();
        }
        $billing->setLastname($last_name);

        $full_name = $firstname . " " . $last_name;

        $country_code = $billing->getCountryId();
        if ($country_code == "-") {
            $country_code = "";
        }
        if ($country_code == "" && isset($params['billing']['country_id'])) {
            $country_code = $params['billing']['country_id'];
        }
        $billing->setCountryId($country_code);

        $billing->save();

        $use_for_shipping = 0;
        if (isset($params['billing']['use_for_shipping'])) {
            $use_for_shipping = $params['billing']['use_for_shipping'];
        }
        if (
            $use_for_shipping == 1 ||
            $billing->getData('same_as_billing') == 1 ||
            $shipping->getData('same_as_billing') == 1
        ) {
            $shipping->setCity($city);
            $city_shipping = $city;
            $shipping->setPostcode($post_code);
            $post_code_shipping = $post_code;
            $shipping->setEmail($email);
            $email_shipping = $email;

            if ($telephone != "") {
                $shipping->setTelephone($telephone);
                $telephone_shipping = $telephone;
            }

            $shipping->setRegion($state);
            $state_shipping = $state;
            $shipping->setRegionId($region_id);
            $region_id_shipping = $region_id;
            $shipping->setStreet(
                array(
                    0 => $street_address0,
                    1 => $street_address1
                )
            );
            $street_address_shipping0 = $street_address0;
            $shipping->setFirstname($firstname);
            $firstname_shipping = $firstname;
            $shipping->setLastname($last_name);
            $last_name_shipping = $last_name;
            $shipping->setCountryId($country_code);
            $country_code_shipping = $country_code;
        } else {
            //Shipping information
            $city_shipping = $shipping->getCity();
            if ($city_shipping == "-") {
                $city_shipping = "";
            }
            if ($city_shipping == "" && isset($params['shipping']['city'])) {
                $city_shipping = $params['shipping']['city'];
            }
            $shipping->setCity($city_shipping);

            $post_code_shipping = $shipping->getPostcode();
            if ($post_code_shipping == "-") {
                $post_code_shipping = "";
            }
            if ($post_code_shipping == "" && isset($params['shipping']['postcode'])) {
                $post_code_shipping = $params['shipping']['postcode'];
            }
            $shipping->setPostcode($post_code_shipping);

            $email_shipping = $shipping->getEmail();
            if ($email_shipping == "-") {
                $email_shipping = "";
            }
            if ($email_shipping == "") {
                $email_shipping = $getCurrentQuote->getCustomerEmail();
            }
            if ($email_shipping == "" && isset($params['billing']['email'])) {
                $email_shipping = $params['billing']['email'];
            }
            $shipping->setEmail($email_shipping);

            $telephone_shipping = $shipping->getTelephone();
            if ($telephone_shipping == "-") {
                $telephone_shipping = "";
            }
            if ($telephone_shipping == "" && isset($params['shipping']['telephone'])) {
                $telephone_shipping = $params['shipping']['telephone'];
            }
            if ($telephone_shipping != "") {
                $shipping->setTelephone($telephone_shipping);
            }

            $state_shipping = $shipping->getRegion();
            if ($state_shipping == "-") {
                $state_shipping = "";
            }
            if ($state_shipping == "" && isset($params['shipping']['region'])) {
                $state_shipping = $params['shipping']['region'];
            }
            $shipping->setRegion($state_shipping);

            $region_id_shipping = $shipping->getRegionId();
            if ($region_id_shipping == "-") {
                $region_id_shipping = "";
            }
            if ($region_id_shipping == "" && isset($params['shipping']['region_id'])) {
                $region_id_shipping = $params['shipping']['region_id'];
            }
            $shipping->setRegionId($region_id_shipping);

            $street_shipping = $shipping->getStreet();
            $street_address_shipping0 = $street_shipping[0];
            if ($street_address_shipping0 == "" && isset($params['shipping']['street'][0])) {
                $street_address_shipping0 = $params['shipping']['street'][0];
            }
            $street_address_shipping1 = "";
            if (isset($street_shipping[1])) {
                $street_address_shipping1 = $street_shipping[1];
                if ($street_address_shipping1 == "" && isset($params['shipping']['street'][1])) {
                    $street_address_shipping1 = $params['shipping']['street'][1];
                }
            }
            $shipping->setStreet(
                array(
                    0 => $street_address_shipping0,
                    1 => $street_address_shipping1
                )
            );

            $firstname_shipping = $shipping->getFirstname();
            if ($firstname_shipping == "-") {
                $firstname_shipping = "";
            }
            if ($firstname_shipping == "" && isset($params['shipping']['firstname'])) {
                $firstname_shipping = $params['shipping']['firstname'];
            }
            if ($firstname_shipping == "") {
                $firstname_shipping = $getCurrentQuote->getCustomerFirstname();
            }
            $shipping->setFirstname($firstname_shipping);

            $last_name_shipping = $shipping->getLastname();
            if ($last_name_shipping == "-") {
                $last_name_shipping = "";
            }
            if ($last_name_shipping == "" && isset($params['shipping']['lastname'])) {
                $last_name_shipping = $params['shipping']['lastname'];
            }
            if ($last_name_shipping == "") {
                $last_name_shipping = $getCurrentQuote->getCustomerLastname();
            }
            $shipping->setLastname($last_name_shipping);

            $country_code_shipping = $shipping->getCountryId();
            if ($country_code_shipping == "-") {
                $country_code_shipping = "";
            }
            if ($country_code_shipping == "" && isset($params['shipping']['country_id'])) {
                $country_code_shipping = $params['shipping']['country_id'];
            }
            $shipping->setCountryId($country_code_shipping);
        }

        $shipping->save();

        $full_name_shipping = $firstname_shipping . " " . $last_name_shipping;
        $merchantReference = $getCurrentQuote->getId();

        $requiredFields = array(
            'consumer_givenNames' => '',
            'consumer_surname' => '',
            'shipping_name' =>  '',
            'shipping_line1' =>  '',
            'shipping_suburb' =>  '',
            'shipping_postcode' =>  '',
            'shipping_countryCode' =>  '',
        );

        $json_array["totalAmount"]["amount"] = "$grandTotal";
        $json_array["totalAmount"]["currency"] = "$currency";
        if (isset($telephone) && $telephone != "") {
            $json_array["consumer"]["phoneNumber"] = $telephone;
        }
        $json_array["consumer"]["givenNames"] = $firstname;
        $requiredFields['consumer_givenNames'] = $firstname;
        $json_array["consumer"]["surname"] = $last_name;
        $requiredFields['consumer_surname'] = $last_name;
        $json_array["consumer"]["email"] = $email;

        //billing info
        $json_array["billing"]["name"] = $full_name;
        if (isset($street_address0) && $street_address0 != "") {
            $json_array["billing"]["line1"] = $street_address0;
        }
        if (isset($city) && $city != "") {
            $json_array["billing"]["suburb"] = $city;
        }
        if (isset($state) && $state != "") {
            $json_array["billing"]["state"] = $state;
        }
        if (isset($post_code) && $post_code != "") {
            $json_array["billing"]["postcode"] = $post_code;
        }
        if (isset($country_code) && $country_code != "") {
            $json_array["billing"]["countryCode"] = $country_code;
        }
        if (isset($telephone) && $telephone != "") {
            $json_array["billing"]["phoneNumber"] = $telephone;
        }

        //shipping info
        $json_array["shipping"]["name"] = $full_name_shipping;
        $requiredFields['shipping_name'] = $full_name_shipping;
        if (isset($street_address_shipping0) && $street_address_shipping0 != "") {
            $json_array["shipping"]["line1"] = $street_address_shipping0;
            $requiredFields['shipping_line1'] = $street_address_shipping0;
        }
        if (isset($city_shipping) && $city_shipping != "") {
            $json_array["shipping"]["suburb"] = $city_shipping;
            $requiredFields['shipping_suburb'] = $city_shipping;
        }
        if (isset($state_shipping) && $state_shipping != "") {
            $json_array["shipping"]["state"] = $state_shipping;
        }
        if (isset($post_code_shipping) && $post_code_shipping != "") {
            $json_array["shipping"]["postcode"] = $post_code_shipping;
            $requiredFields['shipping_postcode'] = $post_code_shipping;
        }
        if (isset($country_code_shipping) && $country_code_shipping != "") {
            $json_array["shipping"]["countryCode"] = $country_code_shipping;
            $requiredFields['shipping_countryCode'] = $country_code_shipping;
        }
        if (isset($telephone_shipping) && $telephone_shipping != "") {
            $json_array["shipping"]["phoneNumber"] = $telephone_shipping;
        }

        foreach ($getAllitems as $item) {
            if ($item->getParentItemId()) {
                continue;
            }
            $scalapay_item = [];
            $name = $item->getName();
            $sku = $item->getSku();
            $qty = $item->getQty();
            $brand = "";
            $gtin = "";
            if ($qty == "") {
                $qty = 1;
            }

            $productId = $item->getProductId();

            $_product = Mage::getModel('catalog/product');
            $_product->load($_product->getIdBySku($sku));
            if ($_product) {
                $this->writeLog("Product id: " . $productId);
                $categoryIds = $_product->getCategoryIds();
                $this->writeLog("category ids: ");
                $this->writeLog($categoryIds);

                $categoryName = "";
                $subCategory  = array();
                $i = 0;

                if (count($categoryIds) > 0) {
                    foreach ($categoryIds as $c) {
                        if ($c > 2) {
                            $category = Mage::getModel('catalog/category')->load($c);

                            if ($i == 0) {
                                $categoryName = $category->getName();
                            } else {
                                $subCategory[] = $category->getName();
                            }

                            $i++;
                        }
                    }
                }

                $scalapay_item["name"] = $name;
                if ($categoryName != "") {
                    $scalapay_item["category"] = $categoryName; // "clothes"
                }
                if (count($subCategory) > 0) {
                    $scalapay_item["subcategory"] = $subCategory; // ["shirt", "long-sleeve"]
                }

                $eavConfig = Mage::getModel('eav/config');
                /* @var $eavConfig Mage_Eav_Model_Config */
                $attributes = $eavConfig->getEntityAttributeCodes(
                    Mage_Catalog_Model_Product::ENTITY,
                    $_product
                );
                if (in_array('brand', $attributes)) {
                    $brand = $_product->getAttributeText('brand');
                }

                $this->writeLog("Brand:" . $brand);
                if (!$brand && in_array('manufacturer', $attributes)) {
                    $brand = $_product->getAttributeText('manufacturer');
                    $this->writeLog("Manufacturer:" . $brand);
                }

                if ($brand) {
                    $scalapay_item["brand"] = $brand; // "TopChoice"
                }
                if ($gtin != "") {
                    $scalapay_item["gtin"] = $gtin; // "123458791330"
                }
            }

            $scalapay_item["sku"] = $sku; // "12341234"
            $scalapay_item["quantity"] = $qty; // 1
            $item_price = $item->getPrice();
            $scalapay_item["price"]["amount"] = "$item_price"; // "10.00"
            $scalapay_item["price"]["currency"] = "$currency"; //"EUR"
            $json_array["items"][] = $scalapay_item;
            $DiscountTotal += $item->getDiscountAmount();
        }

        $isAddressProblem = false;

        foreach ($requiredFields as $key => $value) {
            $this->writeLog("Checking Address field key:" . $key . " => " . $value);
            if ($value == '') {
                $isAddressProblem = true;
                $this->writeLog("Address field value missing: key:" . $key . " => " . $value);
                break;
            }
        }

        if ($isAddressProblem) {
            $return_error_arrays = [];
            $return_error_arrays["error"] = 1;
            $return_error_arrays["message"] = $this->__("There is a problem with required data to place the order.");
            $this->writeLog("address error:");
            $this->writeLog($return_error_arrays);
            return $return_error_arrays;
        }

        if (isset($DiscountTotal) && $DiscountTotal > 0) {
            $discount_displayName = $getCurrentQuote->getCouponCode();
            if ($discount_displayName == "") {
                $discount_displayName = "Discount";
            }

            $discounts_item = [];
            $discounts_item["displayName"] = "$discount_displayName";
            $discounts_item["amount"]["amount"] = "$DiscountTotal";
            $discounts_item["amount"]["currency"] = "$currency";
            $json_array["discounts"][] = $discounts_item;
        }

        $taxAmount = $getCurrentQuote->getTaxAmount();
        if ($taxAmount == "" || $taxAmount == 0) {
            $taxAmount = $getCurrentQuote->getShippingAddress()->getTaxAmount();
        }

        if ($taxAmount == "") {
            $taxAmount = 0;
        }

        $json_array["taxAmount"]["amount"] = "$taxAmount";
        $json_array["taxAmount"]["currency"] = "$currency";
        $confirm_url = Mage::helper("adminhtml")->getUrl("scalapay/index/confirm") .
            "?cart_id=" . $merchantReference .
            "&total=" . $grandTotal;
        $cancel_url = Mage::helper("adminhtml")->getUrl("scalapay/index/cancel");
        $json_array["merchant"]["redirectConfirmUrl"] = "$confirm_url";
        $json_array["merchant"]["redirectCancelUrl"] = "$cancel_url";
        ///$json_array["merchantReference"] = $merchantReference;
        $shippingAmount = $getCurrentQuote->getShippingAmount();

        if ($shippingAmount == "" || $shippingAmount == 0) {
            $shippingAmount = $getCurrentQuote->getShippingAddress()->getShippingAmount();
        }

        if ($shippingAmount == "") {
            $shippingAmount = 0;
        }

        $json_array["shippingAmount"]["amount"] = "$shippingAmount";
        $json_array["shippingAmount"]["currency"] = "$currency";
        $mageVersion = Mage::getVersion();
        $scalapayVersion = Mage::getConfig()->getNode('modules/Scalapay_Scalapay/version');

        $json_array["pluginDetails"]["pluginVersion"] = "$scalapayVersion";
        $json_array["pluginDetails"]["platformVersion"] = "$mageVersion";
        $json_array["pluginDetails"]["platform"] = "magento";
        $json_array["pluginDetails"]["customized"] = "0";
        if ($paymentMethod == "payin4") {
            $json_array["type"] = "online";
            $json_array["product"] = "pay-in-4";
            $json_array["frequency"]["frequencyType"] = "monthly";
            $json_array["frequency"]["number"] = "1";
        } elseif ($paymentMethod == "paylater") {
            $json_array["type"] = "online";
            $json_array["product"] = "later";
            $json_array["frequency"]["frequencyType"] = "daily";
            $json_array["frequency"]["number"] = $this->getPaylaterConfigData("number_of_days");
        } else {
            $json_array["type"] = "online";
            $json_array["product"] = "pay-in-3";
            $json_array["frequency"]["frequencyType"] = "monthly";
            $json_array["frequency"]["number"] = "1";
        }

        $data = $json_array;
        $this->writeLog("data after process: ");
        $this->writeLog($data);
        $return_array = [];
        $urlKey = "v2/orders";
        $result = $this->getCurlHandler($urlKey, $data, "POST");
        $error = $result["error"];
        $response = $result["response"];

        if ($error) {
            $return_array["error"] = 1;
            $return_array["message"] = "cURL Error #:" . $error;
        } else {
            if (isset($response) && $this->isJSON($response)) {
                $res = json_decode($response, true);
            }
            $this->writeLog("Response: ");
            $this->writeLog($res);
            //if error
            if (isset($res["errors"])) {
                $return_array["error"] = 1;
                $errors = "";
                foreach ($res["errors"] as $error) {
                    if (isset($error["messages"][0])) {
                        $errors .= $error["messages"][0] . "\n";
                    }
                }
                $return_array["message"] = $errors;
            }
            //if approved
            if (isset($res["token"]) && $res["token"] != "") {
                $return_array["error"] = 0;
                $return_array["message"] = $res["checkoutUrl"];
            }
        }

        return $return_array;
    }

    /**
     * Get order object
     *
     * @return object
     * @empty false
     *
     */
    public function getOrder()
    {
        $getCurrentQuote = Mage::getSingleton('checkout/session')->getQuote();
        if ($getCurrentQuote) {
            return $getCurrentQuote;
        }

        return false;
    }

    /**
     * Get Quote object
     *
     * @return object
     * @empty false
     *
     */
    public function getQuote()
    {
        $getCurrentQuote = Mage::getSingleton('checkout/session')->getQuote();
        if ($getCurrentQuote) {
            return $getCurrentQuote;
        }

        return false;
    }

    /**
     * Get order shipping info
     *
     * @return object
     * @empty false
     *
     */
    public function getShippingInfo()
    {
        $order = $this->getOrder();
        if ($order) {
            $address = $order->getShippingAddress();

            return $address;
        }

        return false;
    }

    /**
     * Get order billing info
     *
     * @return object
     * @empty false
     *
     */
    public function getBillingInfo()
    {
        $order = $this->getOrder();
        if ($order) {
            $address = $order->getBillingAddress();

            return $address;
        }

        return false;
    }

    /**
     * Send  merchent refference to  Scalapay after order success
     *
     * @param string|null $orderToken
     * @param string|null $merchantOrder
     *
     * @return bool
     */
    public function scalapayUpdateMerchentRefAfterOrder($orderToken, $merchantOrder)
    {
        $urlKey = "v2/orders/" . $orderToken;
        $data = ["merchantReference" => $merchantOrder];
        $result = $this->getCurlHandler($urlKey, $data, "POST");

        return true;
    }

    /**
     * Read response for Scalapay curl request while order Capture
     *
     * @param string|null $orderToken
     *
     * @return array
     */
    public function scalapayCapture($orderToken, $merchantReference)
    {
        $urlKey = "v2/payments/capture";
        $return_array = [];
        $data = ["token" => $orderToken, "merchantReference" => $merchantReference];
        $result = $this->getCurlHandler($urlKey, $data, "POST");
        $error = $result["error"];
        $response = $result["response"];

        if ($error) {
            $return_array["error"] = 1;
            $return_array["message"] = "cURL Error #:" . $error;
        } else {
            if (isset($response) && $this->isJSON($response)) {
                $res = json_decode($response, true);
            }
            //if error
            if (isset($res["errorCode"]) && $res["errorCode"] != "") {
                $return_array["error"] = 1;
                $return_array["message"] = $res["message"];
            }
            //if approved
            if (isset($res["status"]) && $res["status"] == "APPROVED") {
                $return_array["error"] = 0;
                $return_array["message"] = $res["status"];
            }
        }

        return $return_array;
    }

    /**
     * Check the status of a payment at Scalapay
     *
     * @param string|null $orderToken
     *
     * @return array
     */
    public function scalapayPaymentStatus($orderToken)
    {
        $urlKey = "v2/payments/" . $orderToken;
        $return_array = [];
        $data = "";
        $result = $this->getCurlHandler($urlKey, $data, "GET");
        $error = $result["error"];
        $response = $result["response"];

        if ($error) {
            $return_array["error"] = 1;
            $return_array["message"] = "cURL Error #:" . $error;
        } else {
            if (isset($response) && $this->isJSON($response)) {
                $res = json_decode($response, true);
            }
            //if error
            if (isset($res["errorCode"]) && $res["errorCode"] != "") {
                $return_array["error"] = 1;
                $return_array["message"] = $res["message"];
            }
            //if approved
            if (isset($res["status"]) && ($res["status"] == "charged" ||   $res["status"] == "partially_refunded")) {
                $return_array["error"] = 0;
                $return_array["status"] = $res["status"];
                $return_array["total_amount"] = $res["totalAmount"];
                $return_array["order_details"] = $res["orderDetails"];
                $return_array["created"] = $res["created"];
            } else {
                $return_array["error"] = 1;
                $return_array["status"] = $res["status"];
                $return_array["total_amount"] = $res["totalAmount"];
                $return_array["order_details"] = $res["orderDetails"];
                $return_array["created"] = $res["created"];
            }
        }

        return $return_array;
    }

    /**
     * Read response for Scalapay curl request while order Capture
     *
     * @param string|null $orderToken
     * @param string|null $refundAmmount
     * @param string|null $currency
     * @return array
     */
    public function scalapayRefundPayment($orderToken, $merchantOrder, $refundAmount, $currency)
    {
        $urlKey = "v2/payments/" . $orderToken . "/refund";
        $return_array = [];
        $data = [
            "refundAmount" => [
                "amount" => "$refundAmount",
                "currency" => "$currency"
            ],
            "merchantReference" => $merchantOrder
        ];
        $result = $this->getCurlHandler($urlKey, $data, "POST");
        $error = $result["error"];
        $response = $result["response"];

        if ($error) {
            $return_array["error"] = 1;
            $return_array["message"] = "cURL Error #:" . $error;
        } else {
            if (isset($response) && $this->isJSON($response)) {
                $res = json_decode($response, true);
                $this->writeLog($res);
            }
            if (isset($res["errorCode"]) && $res["errorCode"] != "") {
                $return_array["error"] = 1;
                $return_array["errorId"] = $res["errorId"];
                $return_array["errorCode"] = $res["errorCode"];
                $return_array["message"] = $res["message"];
            }
            if (isset($res["refundToken"])) {
                $return_array["error"] = 0;
                $return_array["refundToken"] = $res["refundToken"];
                $return_array["refundedAt"] = $res["refundedAt"];
            }
        }

        return $return_array;
    }

    /**
     * get the payload of a payment at Scalapay
     *
     * @param string|null $orderToken
     *
     * @return array
     */
    public function scalapayGetPayloadData($orderToken)
    {
        $return_array = [];
        $urlKey = "v2/payments/" . $orderToken;
        $data = "";
        $result = $this->getCurlHandler($urlKey, $data, "GET");
        $error = $result["error"];
        $response = $result["response"];

        $return_array["error"] = 1;
        $return_array["billing"] = "";
        $return_array["shipping"] = "";

        if ($error) {
            $return_array["error"] = 1;
            $return_array["message"] = "cURL Error #:" . $error;
        } else {
            if (isset($response) && $this->isJSON($response)) {
                $res = json_decode($response, true);
            }
            //if approved
            if ($res["orderDetails"] !== null) {
                $return_array["error"] = 0;
                $return_array["billing"] = $res["orderDetails"]["billing"];
                $return_array["shipping"] = $res["orderDetails"]["shipping"];
                $return_array["consumer"] = $res["orderDetails"]["consumer"];
            }
        }

        return $return_array;
    }

    /**
     * Send  merchent refference to  Scalapay after order success
     *
     * @param string $urlKey
     * @param array|null $data
     * @param string|null $method
     *
     * @return array
     */
    protected function getCurlHandler($urlKey, $data, $method = "POST")
    {
        $this->getScalapayConnection();
        $url = $this->ws_url . "/" . $urlKey;
        $password = $this->ws_password;
        $authorization = "Authorization: Bearer " . $password;
        $json_array_encoded = "";
        if (isset($data)) {
            $json_array_encoded = json_encode($data);
        }

        $curl = curl_init();

        $return_array = [];
        $curlOptions = [
            CURLOPT_URL => "$url",
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_HTTPHEADER => [
                "Accept: application/json",
                "Content-Type: application/json",
                $authorization
            ]
        ];

        if (strtolower($method) == "post") {
            $curlOptions[CURLOPT_CUSTOMREQUEST] = "POST";
            $curlOptions[CURLOPT_POSTFIELDS] = "$json_array_encoded";
        } elseif (strtolower($method) == "get") {
            $curlOptions[CURLOPT_CUSTOMREQUEST] = "GET";
        } else {
            return ["error" => "wrong curl method", "response" => ""];
        }

        curl_setopt_array($curl, $curlOptions);
        $response = curl_exec($curl);
        $error = curl_error($curl);
        curl_close($curl);
        $result = [
            'response' => $response,
            'error' => $error
        ];

        return $result;
    }

    /**
     * check if string is Json
     *
     * @return True/False
     *
     */
    public function isJSON($string)
    {
        return is_string($string) &&
        is_array(json_decode($string, true)) &&
        (json_last_error() == JSON_ERROR_NONE) ? true : false;
    }

    /**
     * Order cancel request
     *
     * @return rediraction
     *
     */
    public function cancelOrder()
    {
        $currenQuoteId = Mage::getSingleton('checkout/session')->getQuoteId();
        $session = Mage::getSingleton('checkout/session');
        $cart = Mage::getSingleton('checkout/cart');
        $session->setQuoteId($currenQuoteId);
        if ($session->getLastRealOrderId()) {
            $incrementId = $session->getLastRealOrderId();
            if (empty($incrementId)) {
                $session->addError($this->__('Your payment failed, Please try again later'));
                $this->_redirect('checkout/cart');
                return;
            }
            $order = Mage::getModel('sales/order')->loadByIncrementId($session->getLastRealOrderId());
            $session->getQuote()->setIsActive(false)->save();
            $session->clear();
            try {
                $order->setActionFlag(Mage_Sales_Model_Order::ACTION_FLAG_CANCEL, true);
                $order->cancel()->save();
            } catch (Mage_Core_Exception $e) {
                Mage::logException($e);
            }
            $items = $order->getItemsCollection();
            foreach ($items as $item) {
                try {
                    $cart->addOrderItem($item);
                } catch (Mage_Core_Exception $e) {
                    $session->addError($this->__($e->getMessage()));
                    Mage::logException($e);
                    continue;
                }
            }

            $cart->save();
            $session->addError($this->__('Your payment failed. Please try again later'));
        }
    }

    /**
     * Cancel order by quote id
     *
     * @param $quote
     *
     * @return void
     * @throws Exception
     */
    public function cancelFailedOrders($quote){
        $order = Mage::getModel('sales/order')
            ->getCollection()
            ->addFieldToFilter('quote_id', $quote->getId())
            ->addFieldToFilter('grand_total', $quote->getGrandTotal())
            ->addFieldToFilter('status', 'pending')
            ->addFieldToFilter('created_at', array('from' => $quote->getCreatedAt(), 'to' => now()))
            ->getFirstItem();

        if (!$order->getId()) {
            $this->writeLog(
                sprintf("Scalapay: Unable to find the failed order to cancel (quote ID %s)", $quote->getId())
            );
            return;
        }

        $orderToCancel = Mage::getModel('sales/order')->load($order->getId());

        if (!in_array($orderToCancel->getPayment()->getMethod(), array('payin3', 'payin4', 'paylater'))) {
            return;
        }
        $orderToCancel->cancel();
        $orderToCancel->addStatusHistoryComment('Order cancelled because of an error during payment flow.');
        $orderToCancel->save();

        $this->writeLog(
            sprintf("Scalapay: Error in creating merchant order - Order set to status cancelled (ID %s)", $order->getId())
        );
    }

    public function getCurrentLanguage()
    {
        $locale = Mage::app()->getLocale()->getLocaleCode();
        $lang = explode("_", $locale);
        $lang_code = $lang[0];

        return $lang_code;
    }

    /**
     * Check if scalapay is active on the basis of
     * country, language, and currency slection,
     * Virtual product,
     * and price limits
     *
     * @param string|int $grandTotal
     * @param array $products
     * @param string|null $currentCountry
     *
     * @return bool
     */
    public function isSclapayActive($grandTotal, $products = array(), $currentCountry = null, $area = null)
    {
        //checking if Scalapay is activated from admin site
        //Also make sure there is no virtual product
        //echo "current Country: ".$currentCountry;
        if (!$this->getIsActiveFromAdmin()) {
            return false;
        }
        $configData = $this->getScalapayConfig();
        $allowIfLessValue = false;
        if (
            $this->getConfigData('product_apply_min_order_value') == 0 &&
            $area == 'product' && $grandTotal < $configData["minimumAmount"]
        ) {
            $allowIfLessValue = true;
        } elseif (
            $this->getConfigData('cart_apply_min_order_value') == 0 &&
            $area == 'cart' &&
            $grandTotal < $configData["minimumAmount"]
        ) {
            $allowIfLessValue = true;
        }
        // checking if order total or product price is with in specified limits.
        if (
            ($grandTotal < $configData["minimumAmount"] && $allowIfLessValue == false) ||
            $grandTotal > $configData["maximumAmount"]
        ) {
            return false;
        }
        if (count($products) > 0) {
            $disabled_categories = $this->getConfigData('disabled_categories');
            $filter_categories = [];
            if (isset($disabled_categories) && $disabled_categories != "") {
                $categories = explode(",", $disabled_categories);
                if (count($categories) > 0) {
                    foreach ($categories as $c) {
                        $category = Mage::getModel('catalog/category')->load($c);
                        $sub_categories = $category->getAllChildren(true);
                        $filter_categories[] = $category->getId();
                        if (count($sub_categories) > 0) {
                            $filter_categories = array_merge($sub_categories, $filter_categories);
                        }
                    }
                }
            }
            foreach ($products as $pid) {
                if ($pid > 0 && $pid != "") {
                    $p = Mage::getModel('catalog/product')->load($pid);
                    if ($p->getId()) {
                        if (count($filter_categories) > 0) {
                            $productCategoryIds = $p->getCategoryIds();
                            $result = array_intersect($productCategoryIds, $filter_categories);
                            if (count($result) > 0) {
                                return false;
                            }
                        }
                        if ($p->getIsVirtual()) {
                            return false;
                        }
                    }
                }
            }
        }
        //checking if Scalapay is applicable From countries
        if ($this->getConfigData('allowspecific') == 1 && $currentCountry !== null) {
            $availableCountries = explode(',', $this->getConfigData('specificcountry'));
            if (!in_array($currentCountry, $availableCountries)) {
                return false;
            }
        }
        $currentLanguage = Mage::app()->getLocale()->getLocaleCode();
        //checking if Scalapay is applicable From languages
        if ($this->getConfigData('allowspecificlanguage') == 1) {
            $availableLanguages = explode(',', $this->getConfigData('specificlanguage'));
            if (!in_array($currentLanguage, $availableLanguages)) {
                return false;
            }
        }
        $currentCurrency = Mage::app()->getStore()->getCurrentCurrencyCode();
        //checking if Scalapay is applicable From Currencies
        if ($this->getConfigData('allowspecificcurrency') == 1) {
            $availableCurrencies = explode(',', $this->getConfigData('specificcurrency'));
            if (!in_array($currentCurrency, $availableCurrencies)) {
                return false;
            }
        }

        return true;
    }


    /**
     * Check if scalapay Pay in 4 is active on the basis of
     * country, language, and currency slection,
     * Virtual product,
     * and price limits
     *
     * @param string|int $grandTotal
     * @param array $products
     * @param string|null $currentCountry
     *
     * @return bool
     */
    public function isPayin4Active($grandTotal, $products = array(), $currentCountry = null, $area = null)
    {
        //checking if Scalapay is activated from admin site
        //Also make sure there is no virtual product
        //echo "current Country: ".$currentCountry;
        if (!$this->getIsActivePayin4FromAdmin()) {
            return false;
        }
        $configData = $this->getPayin4Config();
        $allowIfLessValue = false;
        if (
            $this->getPayin4ConfigData('product_apply_min_order_value') == 0 &&
            $area == 'product' &&
            $grandTotal < $configData["minimumAmount"]
        ) {
            $allowIfLessValue = true;
        } elseif (
            $this->getPayin4ConfigData('cart_apply_min_order_value') == 0 &&
            $area == 'cart' &&
            $grandTotal < $configData["minimumAmount"]
        ) {
            $allowIfLessValue = true;
        }
        // checking if order total or product price is with in specified limits.
        if (
            ($grandTotal < $configData["minimumAmount"] && $allowIfLessValue == false) ||
            $grandTotal > $configData["maximumAmount"]
        ) {
            return false;
        }
        if (count($products) > 0) {
            $disabled_categories = $this->getPayin4ConfigData('disabled_categories');
            $filter_categories = [];
            if (isset($disabled_categories) && $disabled_categories != "") {
                $categories = explode(",", $disabled_categories);
                if (count($categories) > 0) {
                    foreach ($categories as $c) {
                        $category = Mage::getModel('catalog/category')->load($c);
                        $sub_categories = $category->getAllChildren(true);
                        $filter_categories[] = $category->getId();
                        if (count($sub_categories) > 0) {
                            $filter_categories = array_merge($sub_categories, $filter_categories);
                        }
                    }
                }
            }
            foreach ($products as $pid) {
                if ($pid > 0 && $pid != "") {
                    $p = Mage::getModel('catalog/product')->load($pid);
                    if ($p->getId()) {
                        if (count($filter_categories) > 0) {
                            $productCategoryIds = $p->getCategoryIds();
                            $result = array_intersect($productCategoryIds, $filter_categories);
                            if (count($result) > 0) {
                                return false;
                            }
                        }
                        if ($p->getIsVirtual()) {
                            return false;
                        }
                    }
                }
            }
        }

        //checking if Scalapay is applicable From countries
        if ($this->getPayin4ConfigData('allowspecific') == 1 && $currentCountry !== null) {
            $availableCountries = explode(',', $this->getPayin4ConfigData('specificcountry'));
            if (!in_array($currentCountry, $availableCountries)) {
                return false;
            }
        }
        $currentLanguage = Mage::app()->getLocale()->getLocaleCode();
        //checking if Scalapay is applicable From languages
        if ($this->getPayin4ConfigData('allowspecificlanguage') == 1) {
            $availableLanguages = explode(',', $this->getPayin4ConfigData('specificlanguage'));
            if (!in_array($currentLanguage, $availableLanguages)) {
                return false;
            }
        }
        $currentCurrency = Mage::app()->getStore()->getCurrentCurrencyCode();
        //checking if Scalapay is applicable From Currencies
        if ($this->getPayin4ConfigData('allowspecificcurrency') == 1) {
            $availableCurrencies = explode(',', $this->getPayin4ConfigData('specificcurrency'));
            if (!in_array($currentCurrency, $availableCurrencies)) {
                return false;
            }
        }

        return true;
    }


    /**
     * Check if scalapay Pay in 4 is active on the basis of
     * country, language, and currency slection,
     * Virtual product,
     * and price limits
     *
     * @param string|int $grandTotal
     * @param array $products
     * @param string|null $currentCountry
     *
     * @return bool
     */
    public function isPaylaterActive($grandTotal, $products = array(), $currentCountry = null, $area = null)
    {
        //checking if Scalapay is activated from admin site
        //Also make sure there is no virtual product
        //echo "current Country: ".$currentCountry;
        if (!$this->getIsActivePaylaterFromAdmin()) {
            return false;
        }
        $configData = $this->getPaylaterConfig();
        $allowIfLessValue = false;
        if (
            $this->getPaylaterConfigData('product_apply_min_order_value') == 0 &&
            $area == 'product' &&
            $grandTotal < $configData["minimumAmount"]
        ) {
            $allowIfLessValue = true;
        } elseif (
            $this->getPaylaterConfigData('cart_apply_min_order_value') == 0 &&
            $area == 'cart' &&
            $grandTotal < $configData["minimumAmount"]
        ) {
            $allowIfLessValue = true;
        }
        // checking if order total or product price is with in specified limits.
        if (
            ($grandTotal < $configData["minimumAmount"] && $allowIfLessValue == false) ||
            $grandTotal > $configData["maximumAmount"]
        ) {
            return false;
        }
        if (count($products) > 0) {
            $disabled_categories = $this->getPaylaterConfigData('disabled_categories');
            $filter_categories = [];
            if (isset($disabled_categories) && $disabled_categories != "") {
                $categories = explode(",", $disabled_categories);
                if (count($categories) > 0) {
                    foreach ($categories as $c) {
                        $category = Mage::getModel('catalog/category')->load($c);
                        $sub_categories = $category->getAllChildren(true);
                        $filter_categories[] = $category->getId();
                        if (count($sub_categories) > 0) {
                            $filter_categories = array_merge($sub_categories, $filter_categories);
                        }
                    }
                }
            }
            foreach ($products as $pid) {
                if ($pid > 0 && $pid != "") {
                    $p = Mage::getModel('catalog/product')->load($pid);
                    if ($p->getId()) {
                        if (count($filter_categories) > 0) {
                            $productCategoryIds = $p->getCategoryIds();
                            $result = array_intersect($productCategoryIds, $filter_categories);
                            if (count($result) > 0) {
                                return false;
                            }
                        }
                        if ($p->getIsVirtual()) {
                            return false;
                        }
                    }
                }
            }
        }

        //checking if Scalapay is applicable From countries
        if ($this->getPaylaterConfigData('allowspecific') == 1 && $currentCountry !== null) {
            $availableCountries = explode(',', $this->getPaylaterConfigData('specificcountry'));
            if (!in_array($currentCountry, $availableCountries)) {
                return false;
            }
        }
        $currentLanguage = Mage::app()->getLocale()->getLocaleCode();
        //checking if Scalapay is applicable From languages
        if ($this->getPaylaterConfigData('allowspecificlanguage') == 1) {
            $availableLanguages = explode(',', $this->getPaylaterConfigData('specificlanguage'));
            if (!in_array($currentLanguage, $availableLanguages)) {
                return false;
            }
        }
        $currentCurrency = Mage::app()->getStore()->getCurrentCurrencyCode();
        //checking if Scalapay is applicable From Currencies
        if ($this->getPaylaterConfigData('allowspecificcurrency') == 1) {
            $availableCurrencies = explode(',', $this->getPaylaterConfigData('specificcurrency'));
            if (!in_array($currentCurrency, $availableCurrencies)) {
                return false;
            }
        }

        return true;
    }

    /**
     * Check if scalapay is enabled from admin site configuration
     *
     * @return bool
     */
    public function getIsActiveFromAdmin()
    {
        return (bool)$this->getConfigData('active');
    }

    /**
     * Check if scalapay is enabled from admin site configuration
     *
     * @return bool
     */
    public function getIsActivePayin4FromAdmin()
    {
        return (bool)$this->getPayin4ConfigData('active');
    }

    /**
     * Check if scalapay Pay later is enabled from admin site configuration
     *
     * @return bool
     */
    public function getIsActivePaylaterFromAdmin()
    {
        return (bool)$this->getPaylaterConfigData('active');
    }

    /**
     * Retrieve information from payment configuration
     *
     * @param string $field
     * @param int|string|null|Mage_Core_Model_Store $storeId
     *
     * @return string
     */
    public function getConfigData($field, $storeId = null)
    {
        $data = null;
        $path = null;

        if (null === $storeId) {
            $storeId = Mage::app()->getStore()->getStoreId();
        }

        if ($field != "") {
            switch ($field) {
                case 'active':
                case 'title':
                case 'order_status':
                case 'allowspecific':
                case 'specificcountry':
                case 'allowspecificlanguage':
                case 'specificlanguage':
                case 'allowspecificcurrency':
                case 'specificcurrency':
                case 'scalapay_failed_payment_error':
                case 'payment_action':
                case 'sort_order':
                case 'active':
                    $path = 'payment/payin3/' . $field;
                    break;
                case 'live_mode':
                case 'test_url':
                case 'test_api_password':
                case 'production_url':
                case 'production_api_password':
                case 'site_url':
                case 'scalapay_logo_css':
                    $path = 'scalapay/settings/' . $field;
                    break;
                case 'min_amount':
                case 'max_amount':
                case 'number_of_payments':
                case 'disabled_categories':
                    $path = 'scalapay/payin3/' . $field;
                    break;
                case 'product_text_size':
                case 'product_logo_size':
                case 'product_price_color':
                case 'product_logo_color':
                case 'product_logo_alignment':
                case 'product_amount_selector_array':
                case 'product_pricebox_selector':
                case 'product_show_widget':
                case 'product_text_position':
                case 'product_show_logo':
                case 'product_apply_min_order_value':
                case 'product_hide_price':
                case 'product_scalapay_enable_below_widget_text':
                case 'product_scalapay_below_widget_text':
                case 'product_currency_position':
                case 'product_currency_display':
                case 'product_theme':
                    $path = 'scalapay/payin3/' . $field;
                    break;
                case 'cart_text_size':
                case 'cart_logo_size':
                case 'cart_price_color':
                case 'cart_logo_color':
                case 'cart_logo_alignment':
                case 'cart_amount_selector_array':
                case 'cart_pricebox_selector':
                case 'cart_show_widget':
                case 'cart_text_position':
                case 'cart_show_logo':
                case 'cart_apply_min_order_value':
                case 'cart_hide_price':
                case 'cart_currency_position':
                case 'cart_currency_display':
                case 'cart_theme':
                    $path = 'scalapay/payin3/' . $field;
                    break;
                case 'checkout_text_size':
                case 'checkout_logo_size':
                case 'checkout_price_color':
                case 'checkout_logo_color':
                case 'checkout_logo_alignment':
                case 'checkout_amount_selector_array':
                case 'checkout_pricebox_selector':
                case 'checkout_show_widget':
                case 'checkout_text_position':
                case 'checkout_show_logo':
                case 'checkout_show_title':
                case 'checkout_apply_min_order_value':
                case 'checkout_hide_price':
                case 'checkout_currency_position':
                case 'checkout_currency_display':
                case 'checkout_theme':
                    $path = 'scalapay/payin3/' . $field;
                    break;

                case 'installment_checkout_description':
                case 'installment_description':
                case 'show_installment_description':
                    $path = 'scalapay/payin3/' . $field;
                    break;

                default:
                    $path = 'payment/payin3/' . $field;
                    break;
            }

            $data = Mage::getStoreConfig($path, $storeId);

            switch ($field) {
                case 'installment_checkout_description':
                case 'installment_description':
                case 'title':
                case 'scalapay_failed_payment_error':
                    $data = $this->__($data);
                    break;
            }
        }

        return $data;
    }

    /**
     * Retrieve information from payment configuration
     *
     * @param string $field
     * @param int|string|null|Mage_Core_Model_Store $storeId
     *
     * @return string
     */
    public function getPayin4ConfigData($field, $storeId = null)
    {
        $data = null;
        $path = null;

        if (null === $storeId) {
            $storeId = Mage::app()->getStore()->getStoreId();
        }

        if ($field != "") {
            switch ($field) {
                case 'active':
                case 'title':
                case 'order_status':
                case 'allowspecific':
                case 'specificcountry':
                case 'allowspecificlanguage':
                case 'specificlanguage':
                case 'allowspecificcurrency':
                case 'specificcurrency':
                case 'scalapay_failed_payment_error':
                case 'payment_action':
                case 'sort_order':
                case 'active':
                    $path = 'payment/payin4/' . $field;
                    break;
                case 'live_mode':
                case 'test_url':
                case 'test_api_password':
                case 'production_url':
                case 'production_api_password':
                case 'site_url':
                case 'scalapay_logo_css':
                    $path = 'scalapay/settings/' . $field;
                    break;
                case 'min_amount':
                case 'max_amount':
                case 'number_of_payments':
                case 'disabled_categories':
                    $path = 'scalapay/payin4/' . $field;
                    break;
                case 'product_text_size':
                case 'product_logo_size':
                case 'product_price_color':
                case 'product_logo_color':
                case 'product_logo_alignment':
                case 'product_amount_selector_array':
                case 'product_pricebox_selector':
                case 'product_show_widget':
                case 'product_text_position':
                case 'product_show_logo':
                case 'product_apply_min_order_value':
                case 'product_hide_price':
                case 'product_scalapay_enable_below_widget_text':
                case 'product_scalapay_below_widget_text':
                case 'product_currency_position':
                case 'product_currency_display':
                case 'product_theme':
                    $path = 'scalapay/payin4/' . $field;
                    break;
                case 'cart_text_size':
                case 'cart_logo_size':
                case 'cart_price_color':
                case 'cart_logo_color':
                case 'cart_logo_alignment':
                case 'cart_amount_selector_array':
                case 'cart_pricebox_selector':
                case 'cart_show_widget':
                case 'cart_text_position':
                case 'cart_show_logo':
                case 'cart_apply_min_order_value':
                case 'cart_hide_price':
                case 'cart_currency_position':
                case 'cart_currency_display':
                case 'cart_theme':
                    $path = 'scalapay/payin4/' . $field;
                    break;
                case 'checkout_text_size':
                case 'checkout_logo_size':
                case 'checkout_price_color':
                case 'checkout_logo_color':
                case 'checkout_logo_alignment':
                case 'checkout_amount_selector_array':
                case 'checkout_pricebox_selector':
                case 'checkout_show_widget':
                case 'checkout_text_position':
                case 'checkout_show_logo':
                case 'checkout_show_title':
                case 'checkout_apply_min_order_value':
                case 'checkout_hide_price':
                case 'checkout_currency_position':
                case 'checkout_currency_display':
                case 'checkout_theme':
                    $path = 'scalapay/payin4/' . $field;
                    break;

                case 'installment_checkout_description':
                case 'installment_description':
                case 'show_installment_description':
                    $path = 'scalapay/payin4/' . $field;
                    break;

                default:
                    $path = 'payment/payin4/' . $field;
                    break;
            }

            $data = Mage::getStoreConfig($path, $storeId);

            switch ($field) {
                case 'installment_checkout_description':
                case 'installment_description':
                case 'title':
                case 'scalapay_failed_payment_error':
                    $data = $this->__($data);
                    break;
            }
        }

        return $data;
    }

    /**
     * Retrieve information from payment configuration
     *
     * @param string $field
     * @param int|string|null|Mage_Core_Model_Store $storeId
     *
     * @return string
     */
    public function getPaylaterConfigData($field, $storeId = null)
    {
        $data = null;
        $path = null;

        if (null === $storeId) {
            $storeId = Mage::app()->getStore()->getStoreId();
        }

        if ($field != "") {
            switch ($field) {
                case 'active':
                case 'title':
                case 'order_status':
                case 'allowspecific':
                case 'specificcountry':
                case 'allowspecificlanguage':
                case 'specificlanguage':
                case 'allowspecificcurrency':
                case 'specificcurrency':
                case 'scalapay_failed_payment_error':
                case 'payment_action':
                case 'sort_order':
                    $path = 'payment/paylater/' . $field;
                    break;
                case 'live_mode':
                case 'test_url':
                case 'test_api_password':
                case 'production_url':
                case 'production_api_password':
                case 'site_url':
                case 'scalapay_logo_css':
                    $path = 'scalapay/settings/' . $field;
                    break;
                case 'min_amount':
                case 'max_amount':
                case 'number_of_payments':
                case 'disabled_categories':
                case 'number_of_days':
                    $path = 'scalapay/paylater/' . $field;
                    break;
                case 'product_text_size':
                case 'product_logo_size':
                case 'product_price_color':
                case 'product_logo_color':
                case 'product_logo_alignment':
                case 'product_amount_selector_array':
                case 'product_pricebox_selector':
                case 'product_show_widget':
                case 'product_text_position':
                case 'product_show_logo':
                case 'product_apply_min_order_value':
                case 'product_hide_price':
                case 'product_scalapay_enable_below_widget_text':
                case 'product_scalapay_below_widget_text':
                case 'product_currency_position':
                case 'product_currency_display':
                case 'product_theme':
                    $path = 'scalapay/paylater/' . $field;
                    break;
                case 'cart_text_size':
                case 'cart_logo_size':
                case 'cart_price_color':
                case 'cart_logo_color':
                case 'cart_logo_alignment':
                case 'cart_amount_selector_array':
                case 'cart_pricebox_selector':
                case 'cart_show_widget':
                case 'cart_text_position':
                case 'cart_show_logo':
                case 'cart_apply_min_order_value':
                case 'cart_hide_price':
                case 'cart_currency_position':
                case 'cart_currency_display':
                case 'cart_theme':
                    $path = 'scalapay/paylater/' . $field;
                    break;
                case 'checkout_text_size':
                case 'checkout_logo_size':
                case 'checkout_price_color':
                case 'checkout_logo_color':
                case 'checkout_logo_alignment':
                case 'checkout_amount_selector_array':
                case 'checkout_pricebox_selector':
                case 'checkout_show_widget':
                case 'checkout_text_position':
                case 'checkout_show_logo':
                case 'checkout_show_title':
                case 'checkout_apply_min_order_value':
                case 'checkout_hide_price':
                case 'checkout_currency_position':
                case 'checkout_currency_display':
                case 'checkout_theme':
                    $path = 'scalapay/paylater/' . $field;
                    break;

                case 'installment_checkout_description':
                case 'installment_description':
                case 'show_installment_description':
                    $path = 'scalapay/paylater/' . $field;
                    break;

                default:
                    $path = 'payment/paylater/' . $field;
                    break;
            }

            $data = Mage::getStoreConfig($path, $storeId);

            switch ($field) {
                case 'installment_checkout_description':
                case 'installment_description':
                case 'title':
                case 'scalapay_failed_payment_error':
                    $data = $this->__($data);
                    break;
            }
        }

        return $data;
    }


    /**
     * Save configuration information in payment configuration
     *
     * @param string $path
     * @param int|string|null $value
     *
     * @return bool
     */
    public function setConfigData($path, $value = null)
    {
        if ($path == '') {
            return false;
        }
        Mage::getModel('core/config')->saveConfig($path, $value);
        Mage::getModel('core/config')->cleanCache();

        return true;
    }
    /**
     * Check if a product is virtual or not.
     *
     * @param int $productId
     *
     * @return bool
     */
    public function getIsVirtualProduct($productId)
    {
        $isVirtual = false;
        if ($productId > 0 && is_numeric($productId)) {
            $_product = Mage::getModel('catalog/product')->load($productId);
            if ($_product->getIsVirtual()) {
                $isVirtual = true;
            }
        }

        return $isVirtual;
    }

    /**
     * Prepare and format Scalapay text.
     *
     * @param sting $areakey
     * @param string $price
     * @param string $logo
     *
     * @return string
     */
    public function getScalapayFormatText(
        $areakey,
        $price,
        $logo = "scalapay-logo-black.png",
        $class = "",
        $styles = ""
    ) {
        $price = $this->getScalapayFormatedPrice($price);
        $nop = $this->getConfigData("number_of_payments");
        $description = $this->getConfigData($areakey);
        $description = $this->__($description);
        $description = str_replace("<price>", $price, $description);
        $description = str_replace("<emoji>", "ðŸ˜Š", $description);
        $description = str_replace("<nop>", $nop, $description);

        $css_text = $this->getConfigData('scalapay_logo_text_css');

        $logo_image_src = Mage::getBaseUrl(Mage_Core_Model_Store::URL_TYPE_SKIN) .
            "frontend/base/default/scalapay/images/" . $logo;
        $img_src = '<img ' .
            ' id="scalapay_text_cart_view" ' .
            ' class="' . $class . '"  ' .
            ' style="' . $styles . '" ' .
            ' src="' . $logo_image_src . '" ' .
            ' />';
        $logo_text = '<span style="' . $css_text . '">' . $img_src . '</span>';
        $description = str_replace("<logo>", $logo_text, $description);

        return $description;
    }

    /**
     * Prepare and format Scalapay text.
     *
     * @param sting $areakey
     * @param string $price
     * @param string $logo
     *
     * @return string
     */
    public function getPayin4FormatText($areakey, $price, $logo = "scalapay-logo-black.png", $class = "", $styles = "")
    {
        $price = $this->getScalapayFormatedPrice($price);
        $nop = $this->getPayin4ConfigData("number_of_payments");
        $description = $this->getPayin4ConfigData($areakey);
        $description = $this->__($description);
        $description = str_replace("<price>", $price, $description);
        $description = str_replace("<emoji>", "ðŸ˜Š", $description);
        $description = str_replace("<nop>", $nop, $description);

        $css_text = $this->getPayin4ConfigData('scalapay_logo_text_css');

        $logo_image_src = Mage::getBaseUrl(Mage_Core_Model_Store::URL_TYPE_SKIN) .
            "frontend/base/default/scalapay/images/" . $logo;
        $img_src = '<img ' .
            ' id="scalapay_text_cart_view" ' .
            ' class="' . $class . '"  ' .
            ' style="' . $styles . '" ' .
            ' src="' . $logo_image_src . '" ' .
            ' />';
        $logo_text = '<span style="' . $css_text . '">' . $img_src . '</span>';
        $description = str_replace("<logo>", $logo_text, $description);

        return $description;
    }

    /**
     * Prepare and format Scalapay text.
     *
     * @param string $areakey
     * @param string $price
     * @param string $logo
     *
     * @return string
     */
    public function getPaylaterFormatText(
        $areakey,
        $price,
        $logo = "scalapay-logo-black.png",
        $class = "",
        $styles = ""
    ) {
        $price = $this->getScalapayFormatedPrice($price);
        $nop = $this->getPaylaterConfigData("number_of_payments");
        $description = $this->getPaylaterConfigData($areakey);
        $description = $this->__($description);
        $description = str_replace("<price>", $price, $description);
        $description = str_replace("<emoji>", "ðŸ˜Š", $description);
        $description = str_replace("<nop>", $nop, $description);

        $css_text = $this->getPaylaterConfigData('scalapay_logo_text_css');

        $logo_image_src = Mage::getBaseUrl(Mage_Core_Model_Store::URL_TYPE_SKIN) .
            "frontend/base/default/scalapay/images/" . $logo;
        $img_src = '<img ' .
            ' id="scalapay_text_cart_view" ' .
            ' class="' . $class . '"  ' .
            ' style="' . $styles . '" ' .
            ' src="' . $logo_image_src . '" ' .
            ' />';
        $logo_text = '<span style="' . $css_text . '">' . $img_src . '</span>';
        $description = str_replace("<logo>", $logo_text, $description);

        return $description;
    }


    /**
     * Prepare and format Scalapay text for Checkout page.
     *
     * @param string $price
     *
     * @return string
     */
    public function getScalapayCheckoutPageText($price)
    {
        $logo = $this->getScalapayCheckoutPageLogo();
        $class = "checkout_scalapay_log_new";
        $styles = " width:90px; display: inline-block; float: none; vertical-align: bottom; margin-left: 5px;";
        return  $this->getScalapayFormatText("installment_checkout_description", $price, $logo, $class, $styles);
    }

    /**
     * Prepare and format Scalapay Payin4 text for Checkout page.
     *
     * @param string $price
     *
     * @return string
     */
    public function getPayin4CheckoutPageText($price)
    {
        $logo = $this->getScalapayCheckoutPageLogo();
        $class = "checkout_scalapay_log_new";
        $styles = " width:90px; display: inline-block; float: none; vertical-align: bottom; margin-left: 5px;";
        return  $this->getPayin4FormatText("installment_checkout_description", $price, $logo, $class, $styles);
    }

    /**
     * Prepare and format Scalapay Payin4 text for Checkout page.
     *
     * @param string $price
     *
     * @return string
     */
    public function getPaylaterCheckoutPageText($price)
    {
        $logo = $this->getScalapayCheckoutPageLogo();
        $class = "checkout_scalapay_log_new";
        $styles = " width:90px; display: inline-block; float: none; vertical-align: bottom; margin-left: 5px;";
        return  $this->getPaylaterFormatText("installment_checkout_description", $price, $logo, $class, $styles);
    }

    /**
     * Prepare and format Scalapay payment description for Checkout page.
     * @return string
     */
    public function getScalapayCheckoutInstallmentDescription()
    {
        $description = $this->getConfigData('installment_description');
        $nop = $this->getConfigData("number_of_payments");

        return str_replace("<nop>", $nop, $description);
    }

    /**
     * Prepare and format Scalapay payment description for Checkout page.
     * @return string
     */
    public function getPayin4CheckoutInstallmentDescription()
    {
        $description = $this->getPayin4ConfigData('installment_description');
        $nop = $this->getPayin4ConfigData("number_of_payments");

        return str_replace("<nop>", $nop, $description);
    }

    /**
     * Prepare and format Scalapay payment description for Checkout page.
     * @return string
     */
    public function getPaylaterCheckoutInstallmentDescription()
    {
        $description = $this->getPaylaterConfigData('installment_description');
        $nop = $this->getPaylaterConfigData("number_of_payments");

        return str_replace("<nop>", $nop, $description);
    }

    /**
     * Rreturn scalapay Checkout  logo.
     *
     * @param string $price
     *
     * @return string
     */
    public function getScalapayCheckoutPageLogo()
    {
        $logo = 'scalapay-logo-black.png';

        return $logo;
    }


    /**
     * Rreturn scalapay logo.
     *
     * @param string $price
     *
     * @return string
     */
    public function getScalapayLogo()
    {
        return 'scalapay-logo-black.png';
    }
    /**
     * Prepare and format Scalapay text for Cart page.
     *
     * @param string $price
     *
     * @return string
     */
    public function getScalapayCartPageText($price)
    {
        $logo = "scalapay-logo-black.png";
        return  $this->getScalapayFormatText("installment_cart_description", $price, $logo);
    }

    /**
     * Prepare and format Scalapay text for product page.
     *
     * @param string $price
     *
     * @return string
     */
    public function getScalapayProductPageText($price)
    {
        $logo = "scalapay-logo-black.png";
        return  $this->getScalapayFormatText("installment_copy_description", $price, $logo);
    }

    /**
     * Prepare and format Scalapay text.
     *
     * @param string $price
     *
     * @return string
     */
    public function getScalapayFormatedPrice($price)
    {
        if ($price) {
            $price = Mage::helper('core')->currency($price, true, false);
        }

        return $price;
    }

    /**
     * check if firecheckout is enabled
     *
     * @return bool true/false
     *
     */
    public function isFirecheckoutEnabled()
    {
        $enabled = false;
        $TM_FireCheckout = Mage::helper('core')->isModuleEnabled("TM_FireCheckout");
        if ($TM_FireCheckout) {
            $enabled = true;
        } else {
            return false;
        }

        if (Mage::getStoreConfig('firecheckout/general/enabled')) {
            $enabled = true;
        } else {
            return false;
        }

        return $enabled;
    }

    /**
     * Compare values by epsilon technique.
     *
     * @param sting|int|float|double $value1
     * @param sting|int|float|double $value2
     *
     * @return bool
     */
    public function epsilonCalculation($value1, $value2)
    {
        $this->writeLog("value 1: " . (float) $value1);
        $this->writeLog("value 2: " . (float) $value2);
        $epsilon = 0.011;
        return abs((float) $value1 - (float) $value2) < $epsilon;
    }

    /**
     * Log writer to track bug
     *
     * @param string|array $log_string
     *
     */
    public function writeLog($log_string)
    {
        if (isset($log_string)) {
            $oldFileName = $this->getLogFileName("-2 months");
            if (file_exists($oldFileName)) {
                unlink($oldFileName);
            }

            $filename = $this->getLogFileName();

            if (is_array($log_string)) {
                Mage::log(print_r($log_string, true), null, $filename, true);
            } else {
                Mage::log($log_string, null, $filename, true);
            }
        }
    }

    /**
     * Log writer to track bug
     *
     * @param string $fileDate
     *
     * @return bool string
     *
     */
    public function getLogFileName($fileDate = "")
    {
        $logDir = Mage::getBaseDir('log');
        $filename = "";
        if (isset($fileDate) && $fileDate !== "") {
            $createdtime = new Zend_Date(strtotime($fileDate));
            $dateFile = Mage::getModel('core/date')->date('Y_m', $createdtime);
            $filename = 'scalapay_' . $dateFile . '.log';
        } else {
            $filename = 'scalapay_' . date('Y_m') . '.log';
        }

        return $filename;
    }
}