<?php

class Scalapay_Scalapay_Model_Payin3 extends Mage_Payment_Model_Method_Abstract
{
    protected $_code = 'payin3';

    /**
    * Info and form blocks
    *
    * @var string
    */
    protected $_formBlockType = 'scalapay_scalapay/form_scalapay';
    protected $_infoBlockType = 'scalapay_scalapay/info';
    protected $_isInitializeNeeded      = true;
    protected $_canUseInternal          = false;
    protected $_canUseForMultishipping  = false;

    protected $_isGateway               = true;
    protected $_canAuthorize            = false;
    protected $_canCapture              = true;
    protected $_canCapturePartial       = true;
    protected $_canRefund               = true;
    protected $_canRefundInvoicePartial = true;
    protected $_canVoid                 = false;
    protected $_canFetchTransactionInfo = true;

    public function capture(Varien_Object $payment, $amount)
    {
        $helper = Mage::helper('scalapay');
        $helper->writeLog("calling from internal capture.");
        $orderToken = Mage::getSingleton('core/session')->getOrderToken();
        $payment
                ->setTransactionId($orderToken)
                ->setIsTransactionClosed(0);
        Mage::getSingleton('core/session')->unsOrderToken();

        return $this;
    }

    public function refund(Varien_Object $payment, $amount)
    {
        $helper = Mage::helper('scalapay');
        $creditmemo = $payment->getCreditmemo();
        $orderToken = $payment->getLastTransId();
        $order = $payment->getOrder();
        $helper->writeLog("refunded scalapay token: " . $orderToken);
        $currency = $order->getOrderCurrencyCode();
        $merchantOrder = $order->getIncrementId();
        $paymentMethod = $payment->getMethodInstance()->getCode();
        $helper->writeLog("paymentMethodName: " . $paymentMethod);

        if ($paymentMethod == $this->_code) {
            if (isset($orderToken) && $orderToken != "") {
                $paymentInfo = $helper->scalapayPaymentStatus($orderToken);
                $helper->writeLog("scalapayPaymentStatus: " . $paymentInfo["status"]);
                if (!$paymentInfo["error"]) {
                    if (
                        ($paymentInfo["status"] == "charged" || $paymentInfo["status"] == "partially_refunded") &&
                        $amount > 0 &&
                        $paymentInfo["total_amount"]["amount"] > 0
                    ) {
                        $refundInfo = $helper->scalapayRefundPayment($orderToken, $merchantOrder, $amount, $currency);
                        if (isset($refundInfo['refundToken'])) {
                            $helper->writeLog("Refunded.");
                            $helper->writeLog($amount . $currency . "\n Refunded id: " . $refundInfo['refundToken']);
                            $creditmemo->setTransactionId($refundInfo['refundToken']);
                            $creditmemo->save();
                        } elseif (isset($refundInfo['errorCode'])) { //errorCode: pre_condition_failed
                            $helper->writeLog("Not Refunded - there is some error.");
                            $helper->writeLog($amount . $currency);
                            $helper->writeLog(
                                "error Id: " . $refundInfo['errorId'] .
                                "\n message: " . $refundInfo['message']
                            );
                        }
                    } elseif ($paymentInfo["status"] == "refunded") {
                        $helper->writeLog("Not Refunded.");
                        $helper->writeLog("Already refunded");
                        $helper->writeLog($paymentInfo["order_details"]);
                    } else {
                        $helper->writeLog("Not Refunded.");
                        $helper->writeLog("Error");
                        $helper->writeLog($paymentInfo["order_details"]);
                    }
                } else {
                    $helper->writeLog("Not Refunded.");
                    $helper->writeLog($paymentInfo['message']);
                }
            }//order token
        }// paymentmethod

        return $this;
    }

    public function getCheckoutRedirectUrl()
    {
        return Mage::getUrl('scalapay/index/index', array('_secure' => true));
    }

    /**
     * Check whether payment method can be used
     * @param Mage_Sales_Model_Quote
     * @return bool
     */
    public function isAvailable($quote = null)
    {
        //check for price range
        $helper = Mage::helper('scalapay');
        $getCurrentQuote = Mage::getSingleton('checkout/session')->getQuote();
        $apiName = Mage::getSingleton('api/server')->getApiName();
        $apiType = Mage::getSingleton('api2/request')->getApiType();

        if (
            $getCurrentQuote->getGrandTotal() == 0 &&
            ($apiName != "" || $apiType != "")
        ) {
            $getCurrentQuote = $quote;
            $helper->writeLog("Access through API: " . $apiName . " " . $apiType);
        }

        $shipping = $getCurrentQuote->getShippingAddress();
        $currentCountry = $shipping->getCountryId();
        $productIds = [];
        $getAllItems = $getCurrentQuote->getAllItems();
        foreach ($getAllItems as $item) {
            $productIds[] = $item->getProductId();
        }
        $totalAmount = $getCurrentQuote->getGrandTotal();

        return $helper->isSclapayActive($totalAmount, $productIds, $currentCountry);
    }

    /**
     * Get initialized flag status
     * @return true
     */
    public function isInitializeNeeded()
    {
        return $this->_isInitializeNeeded;
    }

    /**
     * Instantiate state and set it to state onject
     * //@param
     * //@param
     */
    public function initialize($paymentAction, $stateObject)
    {
        $state = Mage_Sales_Model_Order::STATE_PENDING_PAYMENT;
        $stateObject->setState($state);
        $stateObject->setStatus('pending');
        $stateObject->setIsNotified(false);
    }
}
