<?php

class Scalapay_Scalapay_IndexController extends Mage_Core_Controller_Front_Action
{
    protected $checkout_triggered;

    /**
     * Construct function
     */
    public function _construct()
    {
        parent::_construct();
        $this->_config = 'redirect';
        $this->checkout_triggered = false;
    }

    /**
     * Read response for Scalapay curl request to get checkout link
     *
     * @return checkout url
     * @error array
     *
     */
    public function indexAction()
    {
        $helper = Mage::helper('scalapay');
        $helper->writeLog("Scalapay: Index Action");
        $this->checkout_triggered = true;
        $params = $this->getRequest()->getParams();
        $helper->writeLog($params);
        $res = $helper->scalapayOrder($params);
        $response = [];

        if (isset($res["error"]) && $res["error"]) {
            if (isset($res["message"]) && $res["message"] != "") {
                $error_message = $res["message"];
            } else {
                $scalapay_failed_payment_error = $helper->getConfigData('scalapay_failed_payment_error');
                if (isset($scalapay_failed_payment_error) && $scalapay_failed_payment_error != "") {
                    $error_message = $scalapay_failed_payment_error;
                } else {
                    $error_message = $this->__('The payment was not successful. Choose another payment method');
                }
            }
            // Response to the
            $response = array(
                'success' => false,
                'message' => $error_message,
                'redirect' => Mage::getUrl('checkout/cart'),
            );
        } elseif (isset($res["error"]) && $res["error"] == 0) {
            if (isset($res["message"])) {
                $url = trim($res["message"]);
                $response = array(
                    'success' => true,
                    'redirect' => $url
                );
            }
        } else {
            $scalapay_failed_payment_error = $helper->getConfigData('scalapay_failed_payment_error');
            if (isset($scalapay_failed_payment_error) && $scalapay_failed_payment_error != "") {
                $error_message = $scalapay_failed_payment_error;
            } else {
                $error_message = $this->__('The payment was not successful. Choose another payment method');
            }

            // Response to the
            $response = array(
                'success' => false,
                'message' => $error_message,
                'redirect' => Mage::getUrl('checkout/cart'),
            );
        }

        if (true) {
            if ($response['success'] == true && $response['redirect'] != "") {
                Mage::app()->getResponse()->setRedirect($response['redirect'])->sendResponse();

                return $this;
            } else {
                if (isset($response['message'])) {
                    Mage::getSingleton("core/session")->addNotice($response['message']);
                }
                $this->_redirect('checkout/cart');

                return $this;
            }
        } else {
            // Return the json response to the browser
            return $this->getResponse()
                ->setHeader('Content-type', 'application/json')
                ->setBody(json_encode($response));
        }
    }

    /**
     * Order cancel request
     *
     * @return rediraction
     *
     */

    public function cancelAction()
    {
        $helper = Mage::helper('scalapay');
        $helper->writeLog("Scalapay: Cancel Action");
        $scalapay_failed_payment_error = $helper->getConfigData('scalapay_failed_payment_error');
        if (isset($scalapay_failed_payment_error) && $scalapay_failed_payment_error != "") {
            Mage::getSingleton("core/session")->addNotice($scalapay_failed_payment_error);
        } else {
            Mage::getSingleton("core/session")
                ->addNotice($this->__('The payment was not successful. Choose another payment method'));
        }

        //if firecheckout
        $helper = Mage::helper('scalapay');
        $magestoreOnestepcheckout = Mage::helper('core')->isModuleEnabled("Magestore_Onestepcheckout");
        $idevOnestepcheckout = Mage::helper('core')->isModuleEnabled("Idev_OneStepCheckout");
        if ($helper->isFirecheckoutEnabled()) {
            $firecheckout_url_path = Mage::getStoreConfig('firecheckout/general/url_path');
            $this->_redirect($firecheckout_url_path);
        } elseif ($magestoreOnestepcheckout || $idevOnestepcheckout) {
            $this->_redirect('onestepcheckout');
        } else {
            $this->_redirect('checkout/onepage');
        }

        return $this;
    }

    /**
     * Order confirmation after reponse from Scalapay
     *
     * @return rediraction to success page with message
     * @error rediraction to failure page with error
     *
     */
    public function confirmAction()
    {
        $helper = Mage::helper('scalapay');
        $helper->writeLog("Scalapay: Confirm Action");
        $orderToken = "";
        $status = "";
        $scalapay_failed_payment_error = $helper->getConfigData('scalapay_failed_payment_error');
        if (isset($scalapay_failed_payment_error) && $scalapay_failed_payment_error != "") {
            $error_message = $scalapay_failed_payment_error;
        } else {
            $error_message = $this->__("The payment was not successful. Choose another payment method");
        }

        $params = $this->getRequest()->getParams();

        if (isset($params["orderToken"])) {
            $orderToken = $params["orderToken"];
        }

        if (isset($params["orderToken"])) {
            $status = $params["status"];
        }
        $helper->writeLog("Scalapay: Confirm Action - order Token: " . $orderToken);
        $helper->writeLog("Scalapay: Confirm Action - status: " . $status);
        $getCurrentQuote = Mage::getSingleton('checkout/session')->getQuote();
        $grandTotal = $getCurrentQuote->getGrandTotal();
        $quoteIncrementId = $getCurrentQuote->getId();
        $helper->writeLog("Scalapay: Confirm Action - grand Total  : " . $grandTotal);
        $helper->writeLog("Scalapay: Confirm Action - quote incrementId: " . $quoteIncrementId);
        $cart_id_return = "";
        $order_total_return = 0;

        if (isset($params["cart_id"])) {
            $cart_id_return = $params["cart_id"];
        }

        if (isset($params["total"])) {
            $order_total_return = $params["total"];
        }

        $proceed_for_order = true;

        //extra check if user has the same cart details for which he/she paid the amount
        if ($cart_id_return == "" || $order_total_return == 0) {
            $proceed_for_order = false;
        }

        if ($cart_id_return != "" && $order_total_return > 0) {
            if (
                $cart_id_return != $quoteIncrementId ||
                !$helper->epsilonCalculation($grandTotal, $order_total_return)
            ) {
                $proceed_for_order = false;
            }
        }

        /********if confirm responce from scalapay*******/
        $order_status_final = $helper->getConfigData('order_status');
        if (isset($order_status_final) && $order_status_final != "" && $orderToken != "") {
            $duplicatedOrders = Mage::getModel('sales/order_invoice')->getCollection()
                    ->addFieldToSelect('entity_id')
                    ->addFieldToFilter('transaction_id', $orderToken);

            if ($duplicatedOrders->getSize() > 0) {
                $proceed_for_order = false;
                // goto success page
                $helper->writeLog("Scalapay: confirm action - duplicated order: " . $orderToken);
                $this->_redirect('checkout/onepage/success', array('_secure' => true));

                return $this;
            }
        }

        if (isset($status) && $status == "SUCCESS" && isset($orderToken) && $proceed_for_order) {
            /********Set order payment status confirm*******/
            //{capture call replaced}

            $quote = Mage::getSingleton('checkout/session')->getQuote();

            foreach ($quote->getAllVisibleItems() as $item) {
                $_product = Mage::getModel('catalog/product')->loadByAttribute('sku', $item->getProduct()->getSku());
                if (isset($_product) && $_product != null && $_product != false) {
                    $helper->writeLog("product sku: " . $_product->getSku());
                    $stock = Mage::getModel('cataloginventory/stock_item')->loadByProduct($_product);
                    if ($stock->getQty() <= 0 && $stock->getManageStock() == 1 && !$stock->getIsInStock()) {
                        $helper->writeLog("qty: " . $stock->getQty() . " - manage_stock: " . $stock->getManageStock());
                        $helper->writeLog("Item is short in order -please check your stock :");
                        $helper->writeLog($stock->getData());
                        $this->_redirect('checkout/cart');

                        return $this;
                    }
                }
            }

            $isNewCustomer = false;
            try {
                switch ($this->getCheckoutMethod()) {
                    case Mage_Checkout_Model_Type_Onepage::METHOD_GUEST:
                        $this->prepareGuestQuote();
                        break;
                    case Mage_Checkout_Model_Type_Onepage::METHOD_REGISTER:
                        $this->prepareNewCustomerQuote();
                        $isNewCustomer = true;
                        break;
                    default:
                        $this->prepareCustomerQuote();
                        break;
                }
            } catch (Exception $e) {
                Mage::getSingleton("core/session")->addNotice($e->getMessage());
                $helper->writeLog("Scalapay: ERROR - in getting order.");
                $helper->writeLog($e->getMessage());
                $this->_redirect('checkout/cart');
                return $this;
            }

            Mage::register('scalapay_order_place_before', "yes");

            // Collect Rates and Set Shipping & Payment Method
            //$quote->collectTotals()->getPayment()->setMethod('scalapay');
            $quote->collectTotals()->save();
            try {
                $service = Mage::getModel('sales/service_quote', $quote);
                $helper->writeLog("Scalapay: before submitting quote.");
                $service->submitAll();
                $helper->writeLog("Scalapay: after submitting quote.");
                $order = $service->getOrder();
                $helper->writeLog("Scalapay: after getting order.");
            } catch (Exception $e) {
                // cancel order if creation failed.
                $helper->cancelFailedOrders($quote);

                //updating the address in quote:
                //checks for registered customer - validate customer data
                $validationErrors = [];
                $quote->getBillingAddress()->setStoreId($quote->getStoreId());
                $billingAddressValidation = $quote->getBillingAddress()->validate();
                $payload = $helper->scalapayGetPayloadData($orderToken);

                if ($billingAddressValidation !== true) {
                    //billing validation failed.
                    $helper->writeLog("Billing data Validation failed");

                    if ($payload["error"] === 0 && $payload['billing'] !== null) {
                        $name = explode(" ", $payload['billing']['name']);

                        if (
                            $name[0] !== null &&
                            $quote->getBillingAddress()->getFirstname() == ""
                        ) {
                            $fname = $name[0];
                            $quote->getBillingAddress()->setFirstname($fname);
                        }
                        if (
                            $name[1] !== null &&
                            $quote->getBillingAddress()->getLastname() == ""
                        ) {
                            $lname = $name[1];
                            $quote->getBillingAddress()->setLastname($lname);
                        }
                        if (
                            $payload['consumer']['email'] !== null &&
                            $quote->getBillingAddress()->getEmail() == ""
                        ) {
                            $quote->getBillingAddress()->setEmail($payload['consumer']['email']);
                        }
                        if (
                            $payload['billing']['line1'] !== null &&
                            $quote->getBillingAddress()->getStreetLine(1) == ""
                        ) {
                            $quote->getBillingAddress()->setStreet($payload['billing']['line1']);
                        }
                        if (
                            $payload['billing']['suburb'] !== null &&
                            $quote->getBillingAddress()->getCity() == ""
                        ) {
                            $quote->getBillingAddress()->setCity($payload['billing']['suburb']);
                        }
                        if (
                            $payload['billing']['suburb'] !== null &&
                            $quote->getBillingAddress()->getRegion() == ""
                        ) {
                            $quote->getBillingAddress()->setRegion($payload['billing']['suburb']);
                        }
                        if (
                            $payload['billing']['postcode'] !== null &&
                            $quote->getBillingAddress()->getPostcode() == ""
                        ) {
                            $quote->getBillingAddress()->setPostcode($payload['billing']['postcode']);
                        }
                        if (
                            $payload['billing']['countryCode'] !== null &&
                            $quote->getBillingAddress()->getCountryId() == ""
                        ) {
                            $quote->getBillingAddress()->setCountryId($payload['billing']['countryCode']);
                        }
                        if (
                            $payload['billing']['phoneNumber'] !== null &&
                            $quote->getBillingAddress()->getTelephone() == ""
                        ) {
                            $quote->getBillingAddress()->setTelephone($payload['billing']['phoneNumber']);
                        }
                        $quote->getBillingAddress()->save();
                    }
                }

                if (!$quote->isVirtual()) {
                    $quote->getShippingAddress()->setStoreId($quote->getStoreId());
                    $validationResultShipping = $quote->getShippingAddress()->validate();

                    if ($validationResultShipping !== true) {
                        $helper->writeLog("Shipping data Validation failed");
                        //shipping validation failed.
                        if ($payload["error"] === 0 && $payload['shipping'] !== null) {
                            $name = explode(" ", $payload['shipping']['name']);

                            if (
                                $name[0] !== null &&
                                $quote->getShippingAddress()->getFirstname() == ""
                            ) {
                                $fname = $name[0];
                                $quote->getShippingAddress()->setFirstname($fname);
                            }
                            if (
                                $name[1] !== null &&
                                $quote->getShippingAddress()->getLastname() == ""
                            ) {
                                $lname = $name[1];
                                $quote->getShippingAddress()->setLastname($lname);
                            }
                            if (
                                $payload['consumer']['email'] !== null &&
                                $quote->getShippingAddress()->getEmail() == ""
                            ) {
                                $quote->getShippingAddress()->setEmail($payload['consumer']['email']);
                            }
                            if (
                                $payload['shipping']['line1'] !== null &&
                                $quote->getShippingAddress()->getStreetLine(1) == ""
                            ) {
                                $quote->getShippingAddress()->setStreet($payload['shipping']['line1']);
                            }
                            if (
                                $payload['shipping']['suburb'] !== null &&
                                $quote->getShippingAddress()->getCity() == ""
                            ) {
                                $quote->getShippingAddress()->setCity($payload['shipping']['suburb']);
                            }
                            if (
                                $payload['shipping']['suburb'] !== null &&
                                $quote->getShippingAddress()->getRegion() == ""
                            ) {
                                $quote->getShippingAddress()->setRegion($payload['shipping']['suburb']);
                            }
                            if (
                                $payload['shipping']['postcode'] !== null &&
                                $quote->getShippingAddress()->getPostcode() == ""
                            ) {
                                $quote->getShippingAddress()->setPostcode($payload['shipping']['postcode']);
                            }
                            if (
                                $payload['shipping']['countryCode'] !== null &&
                                $quote->getShippingAddress()->getCountryId() == ""
                            ) {
                                $quote->getShippingAddress()->setCountryId($payload['shipping']['countryCode']);
                            }
                            if (
                                $payload['shipping']['phoneNumber'] !== null &&
                                $quote->getShippingAddress()->getTelephone() == ""
                            ) {
                                $quote->getShippingAddress()->setTelephone($payload['shipping']['phoneNumber']);
                            }
                            $quote->getShippingAddress()->save();
                        }
                    }
                }
                /// end validation

                $helper->writeLog("Submitting Quote 2nd time to convert order.");
                if ($payload['consumer']['email'] !== null && $quote->getCustomerEmail() == "") {
                    $helper->writeLog("customer email is empty.");
                    $quote->setCustomerEmail($payload['consumer']['email']);
                }

                try {
                    $helper->writeLog("Scalapay: Trying 2nd time submitting quote.");
                    $service = Mage::getModel('sales/service_quote', $quote);
                    $helper->writeLog("Scalapay: 2nd before submitting quote.");
                    $service->submitAll();
                    $helper->writeLog("Scalapay: 2nd after submitting quote.");
                    $order = $service->getOrder();
                    $helper->writeLog("Scalapay: 2nd after getting order.");
                } catch (Exception $e) {
                    Mage::getSingleton("core/session")->addNotice('Error in creating order, please try again.');
                    $helper->writeLog("Scalapay: 2nd time ERROR - in order creation.");
                    $helper->writeLog($e->getMessage());

                    // cancel order if creation failed.
                    $helper->cancelFailedOrders($quote);
                    $this->_redirect('checkout/cart');

                    return $this;
                }
            }

            if ($order) {
                Mage::dispatchEvent(
                    'checkout_type_onepage_save_order_after',
                    array('order' => $order, 'quote' => $quote)
                );

                try {
                    if (method_exists($order, 'queueNewOrderEmail')) {
                        $helper->writeLog("Scalapay: queueNewOrderEmail exists.");
                        $order->queueNewOrderEmail();
                    } else {
                        $helper->writeLog("Scalapay: queueNewOrderEmail not exists.");
                        $order->sendNewOrderEmail();
                    }
                } catch (Exception $e) {
                    $helper->writeLog("Scalapay: queueNewOrderEmail exception.");
                    Mage::logException($e);
                }
            }

            Mage::dispatchEvent(
                'checkout_submit_all_after',
                array('order' => $order, 'quote' => $quote)
            );

            $incrementId = $order->getIncrementId();
            if (isset($incrementId)) {
                //checking order total is matched with Scalapay order total
                if ($helper->epsilonCalculation($order->getGrandTotal(), $order_total_return)) {
                    $order = Mage::getModel('sales/order')->loadByIncrementId($incrementId);
                    $historyItem = Mage::getModel('sales/order_status_history')
                        ->setOrder($order)
                        ->setStatus($order->getStatus())
                        ->setIsCustomerNotified(true)
                        ->setComment('Gateway has authorized the payment. Order reference id ' . $orderToken)
                        ->setData('entity_name', Mage_Sales_Model_Order::HISTORY_ENTITY_NAME);

                    $historyItem->save();
                    $order->addStatusHistory($historyItem);
                    $order->save();
                    $helper->writeLog("Scalapay: Confirm Action - order created");
                    ////////////$helper->scalapayUpdateMerchentRefAfterOrder($orderToken, $incrementId);
                    /********Capture Payment*******/
                    $merchantReference = $incrementId;
                    $captureResponse = $helper->scalapayCapture($orderToken, $merchantReference);

                    /* if capture fails*/
                    if (isset($captureResponse["error"]) && $captureResponse["error"]) {
                        if (isset($captureResponse["message"])) {
                            Mage::getSingleton("core/session")->addNotice($error_message);
                        } else {
                            Mage::getSingleton("core/session")->addNotice($error_message);
                        }
                        $helper->writeLog(
                            "Scalapay: Confirm Action - capture response error: " .
                            $captureResponse["error"] . " - message: " . $captureResponse["message"]
                        );
                        try {
                            $order->setState(Mage_Sales_Model_Order::STATE_CANCELED, true, 'Cancel Transaction.');
                            $order->setStatus("canceled");
                            $order->save();
                            $helper->writeLog("Scalapay: Confirm Action - order canceled 1");
                        } catch (Exception $e) {
                            $helper->writeLog(
                                "Scalapay: Confirm Action - error in order deleted - " . $e->getMessage()
                            );
                        }

                        //if firecheckout
                        if ($helper->isFirecheckoutEnabled()) {
                            $firecheckout_url_path = Mage::getStoreConfig('firecheckout/general/url_path');
                            $this->_redirect($firecheckout_url_path);
                        } else {
                            $this->_redirect('checkout/onepage');
                        }

                        return $this;
                    } elseif (!$captureResponse["error"]) {  /* if capture got positive response*/
                        $helper->writeLog(
                            "Scalapay: Confirm Action - capture response message: " . $captureResponse["message"]
                        );

                        /* if capture APPROVED*/

                        if (isset($captureResponse["message"]) && $captureResponse["message"] == "APPROVED") {
                            /********Set order payment status approved*******/
                            // add log here
                            $helper->writeLog(
                                "Scalapay: Confirm Action - capture response: " . $captureResponse["message"]
                            );

                            try {
                                if (!$order->canInvoice()) {
                                    $helper->writeLog("Scalapay: Confirm Action - invoice cannot created");
                                    Mage::throwException(Mage::helper('core')->__('Cannot create an invoice.'));
                                }

                                Mage::getSingleton('core/session')->setOrderToken($orderToken);
                                $invoice = Mage::getModel('sales/service_order', $order)->prepareInvoice();
                                if (!$invoice->getTotalQty()) {
                                    Mage::throwException(
                                        Mage::helper('core')->__('Cannot create an invoice without products.')
                                    );
                                }
                                $invoice->setRequestedCaptureCase(Mage_Sales_Model_Order_Invoice::CAPTURE_ONLINE);
                                $invoice->register();
                                $transactionSave = Mage::getModel('core/resource_transaction')
                                    ->addObject($invoice)
                                    ->addObject($invoice->getOrder());

                                $invoice->setTransactionTd($orderToken);
                                $invoice->save();

                                $helper->writeLog("Scalapay: Confirm Action - invoice created: ");
                                $transactionSave->save();
                                $helper->writeLog(
                                    "order token getting from session: " .
                                    Mage::getSingleton('core/session')->getOrderToken()
                                );
                            } catch (Mage_Core_Exception $e) {
                                $helper->writeLog("Scalapay: Confirm Action - exception: " . $e->getMessage());
                            }

                            //update order status
                            $order_status_final = $helper->getConfigData('order_status');
                            if (isset($order_status_final) && $order_status_final != "") {
                                $order->setStatus($order_status_final);
                            }

                            $order->save();
                            $helper->writeLog("Scalapay: Confirm Action - all order information saved");

                            /*****************IF all OK ,redirect to success page start*********************/
                            $session = Mage::getSingleton('checkout/session');
                            $session->setLastQuoteId($session->getQuote()->getId())
                                ->setLastSuccessQuoteId($session->getQuote()->getId());

                            $quoteId = $session->getQuote()->getId();
                            $session = Mage::getSingleton('checkout/type_onepage')->getCheckout();
                            $orderId = $order->getId();
                            $session->setLastSuccessQuoteId($quoteId);
                            $session->setLastQuoteId($quoteId);
                            // Required, otherwise getOrderId() is empty on success.phtml
                            $session->setLastOrderId($orderId);
                            $session->setLastRealOrderId($incrementId);
                            $quote->setIsActive(0);
                            $quote->save();

                            // Now the redirect supposed to work
                            $this->_redirect('checkout/onepage/success', array('_secure' => true));

                            return $this;

                        /*****IF all OK ,redirect to success page end*****/
                        } else { // if capture not approved
                            try {
                                $order->setState(Mage_Sales_Model_Order::STATE_CANCELED, true, 'Cancel Transaction.');
                                $order->setStatus("canceled");
                                $order->save();
                                $helper->writeLog("Scalapay: Confirm Action - order canceled");
                            } catch (Exception $e) {
                                $helper->writeLog(
                                    "Scalapay: Confirm Action - error in order deleted - " . $e->getMessage()
                                );
                            }

                            if (isset($captureResponse["message"])) {
                                Mage::getSingleton("core/session")->addNotice($error_message);
                            } else {
                                Mage::getSingleton("core/session")->addNotice($error_message);
                            }

                            $helper->writeLog("Scalapay: Confirm Action - error 1: " . $error_message);

                            //if firecheckout
                            $helper = Mage::helper('scalapay');
                            if ($helper->isFirecheckoutEnabled()) {
                                $firecheckout_url_path = Mage::getStoreConfig('firecheckout/general/url_path');
                                $this->_redirect($firecheckout_url_path);
                            } else {
                                $this->_redirect('checkout/onepage');
                            }

                            return $this;
                        }
                    }// catptured not any error
                } else {
                    $helper->writeLog("Scalapay: Confirm Action: Scalapay order total is not matched.");
                    $helper->writeLog("Magento  order total: " . $order->getGrandTotal());
                    $helper->writeLog("Scalapay order total: " . $order_total_return);
                    Mage::getSingleton("core/session")
                    ->addNotice("The payment was not successful. Please try again.");
                    $this->_redirect('checkout/cart');
                }
            } else {  /* if issue in order creation process*/

                /********************ISSUE IN ORDER CERATION START************************/

                $helper->writeLog("Scalapay: Confirm Action - error: Order was not created");

                Mage::getSingleton("core/session")->addNotice($error_message);
                $helper->writeLog("Scalapay: Confirm Action - error 3: " . $error_message);

                //if firecheckout
                $helper = Mage::helper('scalapay');
                if ($helper->isFirecheckoutEnabled()) {
                    $firecheckout_url_path = Mage::getStoreConfig('firecheckout/general/url_path');
                    $this->_redirect($firecheckout_url_path);
                } else {
                    $this->_redirect('checkout/onepage');
                }

                return $this;
            }

            /**********************ISSUE IN ORDER CERATION END************************/
        } else {
            Mage::getSingleton("core/session")->addNotice($error_message);
            $helper->writeLog("Scalapay: Confirm Action - error 2: " . $error_message);

            //if firecheckout
            $helper = Mage::helper('scalapay');
            if ($helper->isFirecheckoutEnabled()) {
                $firecheckout_url_path = Mage::getStoreConfig('firecheckout/general/url_path');
                $this->_redirect($firecheckout_url_path);
            } else {
                $this->_redirect('checkout/onepage');
            }

            return $this;
        }

        $this->_redirect('checkout/onepage/success', array('_secure' => true));

        return $this;
    }

    /**
     * Get updated quote total
     *
     * @return string
     *
     */
    public function updatedTotalAction()
    {
        $helper = Mage::helper('scalapay');
        $configData = $helper->getScalapayConfig();
        $numberOfPayments = $configData["numberOfPayments"];
        $total = Mage::getSingleton('checkout/session')->getQuote()->getGrandTotal();
        $total = $total / $numberOfPayments;
        $result = Mage::app()->getStore()->formatPrice($total, false);
        $this->getResponse()->clearHeaders()->setHeader('Content-type', 'application/json', true);
        //Encode Result in JSON
        $response['total'] = $result;

        return $this->getResponse()->setBody(Mage::helper('core')->jsonEncode($response));
    }

    /**
     * country check to show/hide scalapay payment method
     *
     * @return bool
     *
     */
    public function selectCountryAction()
    {
        $helper = Mage::helper('scalapay');
        $params = Mage::app()->getRequest()->getParams();
        if (!empty($params['country_id'])) {
            if (null === $storeId) {
                $storeId = Mage::app()->getStore()->getStoreId();
            }

            $path = 'payment/scalapay/allowspecific';
            $allowspecific = Mage::getStoreConfig($path, $storeId);
            $path = 'payment/scalapay/specificcountry';
            $specificcountry = Mage::getStoreConfig($path, $storeId);
            $show = true;

            if ($allowspecific == 1) {
                $availableCountries = explode(',', $specificcountry);
                if (!in_array($params['country_id'], $availableCountries)) {
                    $show = false;
                }
            }

            //check for virtual products
            $getCurrentQuote = Mage::getSingleton('checkout/session')->getQuote();
            $IsVirtual = false;
            $getAllitems = $getCurrentQuote->getAllItems();

            foreach ($getAllitems as $item) {
                $id = $item->getProductId();
                if ($helper->getIsVirtualProduct($id)) {
                    $IsVirtual = true;
                    break;
                }
            }

            if ($IsVirtual) {
                $show = false;
            }
            //check for price range
            $helper = Mage::helper('scalapay');
            $configData = $helper->getScalapayConfig();
            $grandTotal = $getCurrentQuote->getGrandTotal();

            if ($grandTotal < $configData["minimumAmount"] || $grandTotal > $configData["maximumAmount"]) {
                $show = false;
            }

            $this->getResponse()->clearHeaders()->setHeader('Content-type', 'application/json', true);
            //Encode Result in JSON
            $response['show'] = $show;

            return $this->getResponse()->setBody(Mage::helper('core')->jsonEncode($response));
        }
    }


    /**
     * update scalapay payment method title price on any ajax request for price change
     *
     * @return string
     *
     */
    public function updatepriceAction()
    {
        $getCurrentQuote = Mage::getSingleton('checkout/session')->getQuote();
        $helper = Mage::helper('scalapay');
        $configData = $helper->getScalapayConfig();
        $numberOfPayments = $configData["numberOfPayments"];
        $grandTotal = $getCurrentQuote->getGrandTotal();
        $installment_price = round($grandTotal / $numberOfPayments, 2);
        $installment_checkout_description = $helper->getScalapayCheckoutPageText($installment_price);

        $this->getResponse()->clearHeaders()->setHeader('Content-type', 'application/json', true);
        //Encode Result in JSON
        $response['text'] = $installment_checkout_description;

        return $this->getResponse()->setBody(Mage::helper('core')->jsonEncode($response));
    }

    /**
     * Prepare quote for guest checkout order submit
     *
     * @return Mage_Checkout_Model_Type_Onepage
     */
    protected function prepareGuestQuote()
    {
        $helper = Mage::helper('scalapay');
        $helper->writeLog("Scalapay: Confirm Action - preparing guest quote");
        $quote = $this->getQuote();
        $quote->setCustomerId(null)
            ->setCustomerEmail($quote->getBillingAddress()->getEmail())
            ->setCustomerIsGuest(true)
            ->setCustomerGroupId(Mage_Customer_Model_Group::NOT_LOGGED_IN_ID);
        return $this;
    }

    /**
     * Get quote checkout method
     *
     * @return string
     */
    public function getCheckoutMethod()
    {
        $helper = Mage::helper('scalapay');
        if ($this->getCustomerSession()->isLoggedIn()) {
            $helper->writeLog(
                "Scalapay: Confirm Action -  METHOD: " . Mage_Checkout_Model_Type_Onepage::METHOD_CUSTOMER
            );
            return Mage_Checkout_Model_Type_Onepage::METHOD_CUSTOMER;
        }
        if (!$this->getQuote()->getCheckoutMethod()) {
            $chelper = Mage::helper('checkout');
            if ($chelper->isAllowedGuestCheckout($this->getQuote())) {
                $helper->writeLog(
                    "Scalapay: Confirm Action -  METHOD: " . Mage_Checkout_Model_Type_Onepage::METHOD_GUEST
                );
                $this->getQuote()->setCheckoutMethod(Mage_Checkout_Model_Type_Onepage::METHOD_GUEST);
            } else {
                $helper->writeLog(
                    "Scalapay: Confirm Action -  METHOD: " . Mage_Checkout_Model_Type_Onepage::METHOD_REGISTER
                );
                $this->getQuote()->setCheckoutMethod(Mage_Checkout_Model_Type_Onepage::METHOD_REGISTER);
            }
        }
        return $this->getQuote()->getCheckoutMethod();
    }

    /**
     * Prepare quote for customer registration and customer order submit
     *
     * @return Mage_Checkout_Model_Type_Onepage
     */
    protected function prepareNewCustomerQuote()
    {
        $helper = Mage::helper('scalapay');
        $helper->writeLog("Scalapay: Confirm Action - preparing New Customer quote");
        $quote      = $this->getQuote();
        $billing    = $quote->getBillingAddress();
        $shipping   = $quote->isVirtual() ? null : $quote->getShippingAddress();

        //$customer = Mage::getModel('customer/customer');
        $customer = $quote->getCustomer();
        /* @var $customer Mage_Customer_Model_Customer */
        $customerBilling = $billing->exportCustomerAddress();
        $customer->addAddress($customerBilling);
        $billing->setCustomerAddress($customerBilling);
        $customerBilling->setIsDefaultBilling(true);
        if ($shipping && !$shipping->getSameAsBilling()) {
            $customerShipping = $shipping->exportCustomerAddress();
            $customer->addAddress($customerShipping);
            $shipping->setCustomerAddress($customerShipping);
            $customerShipping->setIsDefaultShipping(true);
        } else {
            $customerBilling->setIsDefaultShipping(true);
        }

        Mage::helper('core')->copyFieldset('checkout_onepage_quote', 'to_customer', $quote, $customer);
        $customer->setPassword($customer->decryptPassword($quote->getPasswordHash()));
        $passwordCreatedTime = $this->getCheckoutSession()
            ->getData('_session_validator_data')['session_expire_timestamp']
            - Mage::getSingleton('core/cookie')->getLifetime();
        $customer->setPasswordCreatedAt($passwordCreatedTime);
        $customer->save();
        $this->getCustomerSession()->loginById($customer->getId());
        $quote->setCustomer($customer)
            ->setCustomerId(true);
        $quote->setPasswordHash('');
    }

    /**
     * Prepare quote for customer order submit
     *
     * @return Mage_Checkout_Model_Type_Onepage
     */
    protected function prepareCustomerQuote()
    {
        $helper = Mage::helper('scalapay');
        $helper->writeLog("Scalapay: Confirm Action - preparing registered Customer quote");
        $quote      = $this->getQuote();
        $billing    = $quote->getBillingAddress();
        $shipping   = $quote->isVirtual() ? null : $quote->getShippingAddress();

        $customer = $this->getCustomerSession()->getCustomer();
        if (!$billing->getCustomerId() || $billing->getSaveInAddressBook()) {
            $customerBilling = $billing->exportCustomerAddress();
            $customer->addAddress($customerBilling);
            $billing->setCustomerAddress($customerBilling);
        }
        if (
            $shipping && !$shipping->getSameAsBilling() &&
            (!$shipping->getCustomerId() || $shipping->getSaveInAddressBook())
        ) {
            $customerShipping = $shipping->exportCustomerAddress();
            $customer->addAddress($customerShipping);
            $shipping->setCustomerAddress($customerShipping);
        }

        if (isset($customerBilling) && !$customer->getDefaultBilling()) {
            $customerBilling->setIsDefaultBilling(true);
        }
        if ($shipping && isset($customerShipping) && !$customer->getDefaultShipping()) {
            $customerShipping->setIsDefaultShipping(true);
        } elseif (isset($customerBilling) && !$customer->getDefaultShipping()) {
            $customerBilling->setIsDefaultShipping(true);
        }
        $quote->setCustomer($customer);
    }

    /**
     * Quote object getter
     *
     * @return Mage_Sales_Model_Quote
     */
    public function getQuote()
    {
        return Mage::getSingleton('checkout/session')->getQuote();
    }

    /**
     * Get customer session object
     *
     * @return Mage_Checkout_Model_Session
     */
    public function getCheckoutSession()
    {
        return Mage::getSingleton('checkout/session');
    }

    /**
     * Get customer session object
     *
     * @return Mage_Customer_Model_Session
     */
    public function getCustomerSession()
    {
        return Mage::getSingleton('customer/session');
    }
}
